# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AccountPerformanceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    TIMEPERIOD = 'TimePeriod'

    CURRENCYCODE = 'CurrencyCode'

    ADDISTRIBUTION = 'AdDistribution'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    AVERAGECPC = 'AverageCpc'

    SPEND = 'Spend'

    AVERAGEPOSITION = 'AveragePosition'

    CONVERSIONS = 'Conversions'

    CONVERSIONRATE = 'ConversionRate'

    COSTPERCONVERSION = 'CostPerConversion'

    LOWQUALITYCLICKS = 'LowQualityClicks'

    LOWQUALITYCLICKSPERCENT = 'LowQualityClicksPercent'

    LOWQUALITYIMPRESSIONS = 'LowQualityImpressions'

    LOWQUALITYIMPRESSIONSPERCENT = 'LowQualityImpressionsPercent'

    LOWQUALITYCONVERSIONS = 'LowQualityConversions'

    LOWQUALITYCONVERSIONRATE = 'LowQualityConversionRate'

    DEVICETYPE = 'DeviceType'

    DEVICEOS = 'DeviceOS'

    IMPRESSIONSHAREPERCENT = 'ImpressionSharePercent'

    IMPRESSIONLOSTTOBUDGETPERCENT = 'ImpressionLostToBudgetPercent'

    IMPRESSIONLOSTTORANKAGGPERCENT = 'ImpressionLostToRankAggPercent'

    PHONEIMPRESSIONS = 'PhoneImpressions'

    PHONECALLS = 'PhoneCalls'

    PTR = 'Ptr'

    NETWORK = 'Network'

    TOPVSOTHER = 'TopVsOther'

    BIDMATCHTYPE = 'BidMatchType'

    DELIVEREDMATCHTYPE = 'DeliveredMatchType'

    ASSISTS = 'Assists'

    REVENUE = 'Revenue'

    RETURNONADSPEND = 'ReturnOnAdSpend'

    COSTPERASSIST = 'CostPerAssist'

    REVENUEPERCONVERSION = 'RevenuePerConversion'

    REVENUEPERASSIST = 'RevenuePerAssist'

    ACCOUNTSTATUS = 'AccountStatus'

    LOWQUALITYGENERALCLICKS = 'LowQualityGeneralClicks'

    LOWQUALITYSOPHISTICATEDCLICKS = 'LowQualitySophisticatedClicks'

    EXACTMATCHIMPRESSIONSHAREPERCENT = 'ExactMatchImpressionSharePercent'

    CUSTOMERID = 'CustomerId'

    CUSTOMERNAME = 'CustomerName'

    CLICKSHAREPERCENT = 'ClickSharePercent'

    ABSOLUTETOPIMPRESSIONSHAREPERCENT = 'AbsoluteTopImpressionSharePercent'

    TOPIMPRESSIONSHARELOSTTORANKPERCENT = 'TopImpressionShareLostToRankPercent'

    TOPIMPRESSIONSHARELOSTTOBUDGETPERCENT = 'TopImpressionShareLostToBudgetPercent'

    ABSOLUTETOPIMPRESSIONSHARELOSTTORANKPERCENT = 'AbsoluteTopImpressionShareLostToRankPercent'

    ABSOLUTETOPIMPRESSIONSHARELOSTTOBUDGETPERCENT = 'AbsoluteTopImpressionShareLostToBudgetPercent'

    TOPIMPRESSIONSHAREPERCENT = 'TopImpressionSharePercent'

    ABSOLUTETOPIMPRESSIONRATEPERCENT = 'AbsoluteTopImpressionRatePercent'

    TOPIMPRESSIONRATEPERCENT = 'TopImpressionRatePercent'

    ALLCONVERSIONS = 'AllConversions'

    ALLREVENUE = 'AllRevenue'

    ALLCONVERSIONRATE = 'AllConversionRate'

    ALLCOSTPERCONVERSION = 'AllCostPerConversion'

    ALLRETURNONADSPEND = 'AllReturnOnAdSpend'

    ALLREVENUEPERCONVERSION = 'AllRevenuePerConversion'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    AUDIENCEIMPRESSIONSHAREPERCENT = 'AudienceImpressionSharePercent'

    AUDIENCEIMPRESSIONLOSTTORANKPERCENT = 'AudienceImpressionLostToRankPercent'

    AUDIENCEIMPRESSIONLOSTTOBUDGETPERCENT = 'AudienceImpressionLostToBudgetPercent'

    AVERAGECPM = 'AverageCpm'

    CONVERSIONSQUALIFIED = 'ConversionsQualified'

    LOWQUALITYCONVERSIONSQUALIFIED = 'LowQualityConversionsQualified'

    ALLCONVERSIONSQUALIFIED = 'AllConversionsQualified'

    VIEWTHROUGHCONVERSIONSQUALIFIED = 'ViewThroughConversionsQualified'

    VIEWTHROUGHREVENUE = 'ViewThroughRevenue'

    VIDEOVIEWS = 'VideoViews'

    VIEWTHROUGHRATE = 'ViewThroughRate'

    AVERAGECPV = 'AverageCPV'

    VIDEOVIEWSAT25PERCENT = 'VideoViewsAt25Percent'

    VIDEOVIEWSAT50PERCENT = 'VideoViewsAt50Percent'

    VIDEOVIEWSAT75PERCENT = 'VideoViewsAt75Percent'

    COMPLETEDVIDEOVIEWS = 'CompletedVideoViews'

    VIDEOCOMPLETIONRATE = 'VideoCompletionRate'

    TOTALWATCHTIMEINMS = 'TotalWatchTimeInMS'

    AVERAGEWATCHTIMEPERVIDEOVIEW = 'AverageWatchTimePerVideoView'

    AVERAGEWATCHTIMEPERIMPRESSION = 'AverageWatchTimePerImpression'

    SALES = 'Sales'

    COSTPERSALE = 'CostPerSale'

    REVENUEPERSALE = 'RevenuePerSale'

    INSTALLS = 'Installs'

    COSTPERINSTALL = 'CostPerInstall'

    REVENUEPERINSTALL = 'RevenuePerInstall'

    DOWNLOADS = 'Downloads'

    POSTCLICKDOWNLOADRATE = 'PostClickDownloadRate'

    COSTPERDOWNLOAD = 'CostPerDownload'

    APPINSTALLS = 'AppInstalls'

    POSTCLICKINSTALLRATE = 'PostClickInstallRate'

    CPI = 'CPI'

    PURCHASES = 'Purchases'

    POSTINSTALLPURCHASERATE = 'PostInstallPurchaseRate'

    CPP = 'CPP'

    SUBSCRIPTIONS = 'Subscriptions'

    POSTINSTALLSUBSCRIPTIONRATE = 'PostInstallSubscriptionRate'

    CPS = 'CPS'

    NEWCUSTOMERCONVERSIONS = 'NewCustomerConversions'

    NEWCUSTOMERREVENUE = 'NewCustomerRevenue'

    NEWCUSTOMERCONVERSIONRATE = 'NewCustomerConversionRate'

    NEWCUSTOMERCPA = 'NewCustomerCPA'

    NEWCUSTOMERRETURNONADSPEND = 'NewCustomerReturnOnAdSpend'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AccountPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))





import re
from .csv_reader import _CsvReader
from bingads.v13.reporting.report_contract import InvalidReportColumnException

class _RowReportRecord:

    def __init__(self, row_values):
        self._row_values = row_values

    def value(self, header):
        flag, result = self._row_values.try_get_value(header)
        if flag:
            return result
        raise InvalidReportColumnException(header)

    def int_value(self, header):
        try:
            return 0 if '--' == self.value(header) else int(self.value(header))
        except ValueError:
            return 0

    def float_value(self, header):
        try:
            return 0 if '--' == self.value(header) else float(self.value(header))
        except ValueError:
            return 0

class _RowReportRecordReader():
    """ Provides a method to read one row from report file"""

    def __init__(self, file_path, delimiter):
        self._column_mapping = None
        self._csv_reader = _CsvReader(file_path, delimiter)

    def read_next_header(self, parser):

        def remove_bom(unicode_str):
            unicode_bom = u'\N{ZERO WIDTH NO-BREAK SPACE}'
            if unicode_str and unicode_str[0] == unicode_bom:
                unicode_str = unicode_str[1:]
            return unicode_str
        try:
            fields =  [remove_bom(header) for header in next(self._csv_reader)] 

            if self.valid_header(fields) == False:
                return self.read_next_header(parser)
            
            valid_values = list(filter(lambda x: x is not None and len(x) > 0, [s.strip() for s in fields]))
            
            if len(valid_values) == 0:
                return False
            else:
                header = parser.parse_header(fields)
                if len(valid_values) > 1:
                    if header:
                        self._column_mapping = dict(zip(fields, range(0, len(fields))))
                        self.peek()
                    elif self.valid_record(fields):
                        # Exclude Column Header is set
                        self.next_object = _RowReportRecord(_RowValues(columns=fields, mappings=None))
                    return False
                return True
        except StopIteration:
            return False

    def valid_header(self, fields):
        if fields is None:
            return False
        
        valid_values = list(filter(lambda x: x is not None and len(x) > 0, [s.replace('-', '').strip() for s in fields]))
        return len(valid_values) > 0
        pass

    def read_next_object(self):
        if self.next_object != None:
            ret = self.next_object
            self.peek()
            return ret
        return None

    def peek(self):
        self.next_object = None
        try:
            row_values = self._read_next_row_values()
            self.next_object = _RowReportRecord(row_values)
        except StopIteration:
            pass

    def valid_record(self, values):
        if values is None:
            return False

        valid_values = list(filter(lambda x: x is not None and len(x) > 0, [s.replace('-', '').strip() for s in values]))

        if len(valid_values) == 0:
            return False
        if re.match(u'Total|\xa9\\d+ Microsoft Corporation. All rights reserved.*', valid_values[0]):
            return False

        return True
    
    def __exit__(self, exc_type, exc_value, traceback):
        self._csv_reader.__exit__(exc_type, exc_value, traceback)

    def close(self):
        self._csv_reader.close()

    def _read_next_row_values(self):
        values = next(self._csv_reader)
        if self.valid_record(values) == False:
            return self._read_next_row_values()
        return _RowValues(columns=values, mappings=self._column_mapping)

    def _read_headers(self):
        pass

class _ReportStreamReader():

    def __init__(self, file_path, file_type):
        self._report_object_reader = _RowReportRecordReader(file_path, ',' if file_type == 'Csv' else '\t')

    def read_report_header(self, parser):
        while (self._report_object_reader.read_next_header(parser) == True):
            pass
        pass

    def read(self):
        peeked = self._report_object_reader.read_next_object()
        if peeked is not None:
            return peeked
        self._report_object_reader.close()
        return None

    def close(self):
        self._report_object_reader.close()

    def __exit__(self, exc_type, exc_value, traceback):
        self._report_object_reader.__exit__(exc_type, exc_value, traceback)

class _RowReportIterator():
    
    def __init__(self, stream_reader):
        self._stream_reader = stream_reader

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._stream_reader.__exit__(exc_type, exc_value, traceback)

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        ret = self._stream_reader.read()
        if not ret:
            raise StopIteration()
        return ret

class _RowValues:
    def __init__(self, mappings=None, columns=None):
        self._mappings = mappings
        self._columns = columns

    def __getitem__(self, key):
        return self.columns[self._mappings[key]]

    def __setitem__(self, key, value):
        self.columns[self._mappings[key]] = value

    def __contains__(self, item):
        return item in self.mappings

    def __len__(self):
        return len(self.mappings)

    def __str__(self):
        return u'{' + u', '.join([u'{0}:{1}'.format(k, self.columns[v]) for (k, v) in self.mappings.items()]) + u'}'

    def try_get_value(self, header):
        if not self.mappings or header not in self.mappings:
            return False, None
        return True, self[header]

    def to_dict(self):
        return dict([(k, self.columns[v]) for (k, v) in self.mappings.items()])

    @property
    def mappings(self):
        return self._mappings

    @property
    def columns(self):
        return self._columns


# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class SecretQuestion(str, Enum):
    NONE = 'None'

    FAVORITEPETSNAME = 'FavoritePetsName'

    FAVORITEMOVIE = 'FavoriteMovie'

    ANNIVERSARY = 'Anniversary'

    FATHERMIDDLENAME = 'FatherMiddleName'

    SPOUSEMIDDLENAME = 'SpouseMiddleName'

    FIRSTCHILDMIDDLENAME = 'FirstChildMiddleName'

    HIGHSCHOOLNAME = 'HighSchoolName'

    FAVORITETEACHERNAME = 'FavoriteTeacherName'

    FAVORITESPORTSTEAM = 'FavoriteSportsTeam'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SecretQuestion from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
from enum import Flag, auto

class DeliveredMatchTypeReportFilter(Flag):
    EXACT = auto()

    PHRASE = auto()

    BROAD = auto()

    EXACTCLOSEVARIANT = auto()

    PHRASECLOSEVARIANT = auto()

	
    def to_json(self) -> str:
        """Convert to JSON string representation"""
        return self._to_str()
	
    def __str__(self) -> str:
        """
        Return a human-readable string representation of the campaign types.
        """
        return self._to_str()

    def _to_str(self) -> str:
        """Convert the enum flags to a comma-separated string of quoted, capitalized names"""
        names = [flag.name.title() for flag in DeliveredMatchTypeReportFilter if flag & self]
        if not names:
            return "None"
        if len(names) == 1:
            # For single values, use single quotes
            return f"{names[0]}"
        # For multiple values, use double quotes with comma separation
        return ", ".join(f"{name}" for name in names)
		
    @classmethod
    def __get_validators__(cls):
        yield cls._validate
		
    @classmethod
    def _validate(cls, value, handler) -> "DeliveredMatchTypeReportFilter":
        if isinstance(value, cls):
            return value
        if isinstance(value, int):
            return cls(value)
        if isinstance(value, str):
            # Handle comma-separated campaign types
            types = [type_str.strip() for type_str in value.split(',')]
            if len(types) > 1:
                result = cls.NONE
                for type_str in types:
                    try:
                        result |= getattr(cls, type_str.upper())
                    except AttributeError:
                        raise ValueError(f"Invalid CampaignType value: {type_str}")
                return result
            else:
                # Single campaign type
                try:
                    return getattr(cls, value.strip().upper())
                except AttributeError:
                    raise ValueError(f"Invalid DeliveredMatchTypeReportFilter value: {value}")
        raise ValueError(f"Cannot convert {value} to DeliveredMatchTypeReportFilter")





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.health_check_column_metadata import HealthCheckColumnMetadata
from typing_extensions import Self

class HealthCheckMetadata(BaseModel):
    """
    HealthCheckMetadata
    """ # noqa: E501
    health_check_name: Optional[StrictStr] = Field(default=None, alias="HealthCheckName")
    health_check_display_name: Optional[StrictStr] = Field(default=None, alias="HealthCheckDisplayName")
    health_check_description: Optional[StrictStr] = Field(default=None, alias="HealthCheckDescription")
    health_check_sub_type: Optional[StrictStr] = Field(default=None, alias="HealthCheckSubType")
    health_check_category: Optional[StrictStr] = Field(default=None, alias="HealthCheckCategory")
    health_check_severity: Optional[StrictStr] = Field(default=None, alias="HealthCheckSeverity")
    health_check_category_display_name: Optional[StrictStr] = Field(default=None, alias="HealthCheckCategoryDisplayName")
    health_check_columns_metadata: Optional[List[Optional[HealthCheckColumnMetadata]]] = Field(default=None, alias="HealthCheckColumnsMetadata")
    __properties: ClassVar[List[str]] = ["HealthCheckName", "HealthCheckDisplayName", "HealthCheckDescription", "HealthCheckSubType", "HealthCheckCategory", "HealthCheckSeverity", "HealthCheckCategoryDisplayName", "HealthCheckColumnsMetadata"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in health_check_columns_metadata (list)
        _items = []
        if self.health_check_columns_metadata:
            for _item_health_check_columns_metadata in self.health_check_columns_metadata:
                if _item_health_check_columns_metadata:
                    _items.append(_item_health_check_columns_metadata.to_dict())
            _dict['HealthCheckColumnsMetadata'] = _items
        # set to None if health_check_name (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_name is None and "health_check_name" in self.model_fields_set:
            _dict['HealthCheckName'] = None

        # set to None if health_check_display_name (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_display_name is None and "health_check_display_name" in self.model_fields_set:
            _dict['HealthCheckDisplayName'] = None

        # set to None if health_check_description (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_description is None and "health_check_description" in self.model_fields_set:
            _dict['HealthCheckDescription'] = None

        # set to None if health_check_sub_type (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_sub_type is None and "health_check_sub_type" in self.model_fields_set:
            _dict['HealthCheckSubType'] = None

        # set to None if health_check_category (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_category is None and "health_check_category" in self.model_fields_set:
            _dict['HealthCheckCategory'] = None

        # set to None if health_check_severity (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_severity is None and "health_check_severity" in self.model_fields_set:
            _dict['HealthCheckSeverity'] = None

        # set to None if health_check_category_display_name (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_category_display_name is None and "health_check_category_display_name" in self.model_fields_set:
            _dict['HealthCheckCategoryDisplayName'] = None

        # set to None if health_check_columns_metadata (nullable) is None
        # and model_fields_set contains the field
        if self.health_check_columns_metadata is None and "health_check_columns_metadata" in self.model_fields_set:
            _dict['HealthCheckColumnsMetadata'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of HealthCheckMetadata from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "HealthCheckName": obj.get("HealthCheckName") if obj.get("HealthCheckName") is not None else None,
                        "HealthCheckDisplayName": obj.get("HealthCheckDisplayName") if obj.get("HealthCheckDisplayName") is not None else None,
                        "HealthCheckDescription": obj.get("HealthCheckDescription") if obj.get("HealthCheckDescription") is not None else None,
                        "HealthCheckSubType": obj.get("HealthCheckSubType") if obj.get("HealthCheckSubType") is not None else None,
                        "HealthCheckCategory": obj.get("HealthCheckCategory") if obj.get("HealthCheckCategory") is not None else None,
                        "HealthCheckSeverity": obj.get("HealthCheckSeverity") if obj.get("HealthCheckSeverity") is not None else None,
                        "HealthCheckCategoryDisplayName": obj.get("HealthCheckCategoryDisplayName") if obj.get("HealthCheckCategoryDisplayName") is not None else None,
                        "HealthCheckColumnsMetadata": [HealthCheckColumnMetadata.from_dict(_item) for _item in obj["HealthCheckColumnsMetadata"]] if obj.get("HealthCheckColumnsMetadata") is not None else None
        })
        return _obj



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *


class BulkAdCustomizerAttributeEntityBase(_SingleRecordBulkEntity):
    """ Represents an AdCustomizerAttributeEntity.
    
    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, id = None, name=None, parent_id=None, attribute_value=None, editorial_status = None):
        super(BulkAdCustomizerAttributeEntityBase, self).__init__()
        self._id = id
        self._name = name
        self._attribute_value = attribute_value
        self._parent_id = parent_id
        self._editorial_status = editorial_status


    @property
    def id(self):
        """ the id of bulk record
        Corresponds to the 'Id' field in the bulk file.

        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, value):
        self._id = value        

    @property
    def name(self):
        """ the name of the ad customizer attribute entity
        Corresponds to the 'Name' field in the bulk file.

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value
         
    @property
    def attribute_value(self):
        """ the value of ad customizer attribute 
        Corresponds to the 'AdCustomizer AttributeValue' field in the bulk file.

        :rtype: str
        """
        return self._attribute_value

    @attribute_value.setter
    def attribute_value(self, value):
        self._attribute_value = value
        
    @property
    def parent_id(self):
        """ the parent id of bulk record
        Corresponds to the 'ParentId' field in the bulk file.

        :rtype: long
        """
        return self._parent_id

    @parent_id.setter
    def parent_id(self, value):
        self._parent_id = value   
          
    @property
    def editorial_status(self):
        """ the editorial status of ad customizer attribute
        Corresponds to the 'Editorial Status' field in the bulk file.

        :rtype: str
        """
        return self._editorial_status

    @editorial_status.setter
    def editorial_status(self, value):
        self._editorial_status = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, 'id', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: bulk_str(c.name),
            csv_to_field=lambda c, v: setattr(c, 'name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdCustomizerAttributeValue,
            field_to_csv=lambda c: bulk_str(c.attribute_value),
            csv_to_field=lambda c, v: setattr(c, 'attribute_value', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.parent_id),
            csv_to_field=lambda c, v: setattr(c, 'parent_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialStatus,
            field_to_csv=lambda c: bulk_str(c.editorial_status),
            csv_to_field=lambda c, v: setattr(c, 'editorial_status', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkAdCustomizerAttributeEntityBase._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkAdCustomizerAttributeEntityBase._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAdCustomizerAttributeEntityBase, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class Address(BaseModel):
    """
    Address
    """ # noqa: E501
    city: Optional[StrictStr] = Field(default=None, alias="City")
    country_code: Optional[StrictStr] = Field(default=None, alias="CountryCode")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    line1: Optional[StrictStr] = Field(default=None, alias="Line1")
    line2: Optional[StrictStr] = Field(default=None, alias="Line2")
    line3: Optional[StrictStr] = Field(default=None, alias="Line3")
    line4: Optional[StrictStr] = Field(default=None, alias="Line4")
    postal_code: Optional[StrictStr] = Field(default=None, alias="PostalCode")
    state_or_province: Optional[StrictStr] = Field(default=None, alias="StateOrProvince")
    time_stamp: Optional[StrictStr] = Field(default=None, alias="TimeStamp")
    business_name: Optional[StrictStr] = Field(default=None, alias="BusinessName")
    __properties: ClassVar[List[str]] = ["City", "CountryCode", "Id", "Line1", "Line2", "Line3", "Line4", "PostalCode", "StateOrProvince", "TimeStamp", "BusinessName"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if city (nullable) is None
        # and model_fields_set contains the field
        if self.city is None and "city" in self.model_fields_set:
            _dict['City'] = None

        # set to None if country_code (nullable) is None
        # and model_fields_set contains the field
        if self.country_code is None and "country_code" in self.model_fields_set:
            _dict['CountryCode'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if line1 (nullable) is None
        # and model_fields_set contains the field
        if self.line1 is None and "line1" in self.model_fields_set:
            _dict['Line1'] = None

        # set to None if line2 (nullable) is None
        # and model_fields_set contains the field
        if self.line2 is None and "line2" in self.model_fields_set:
            _dict['Line2'] = None

        # set to None if line3 (nullable) is None
        # and model_fields_set contains the field
        if self.line3 is None and "line3" in self.model_fields_set:
            _dict['Line3'] = None

        # set to None if line4 (nullable) is None
        # and model_fields_set contains the field
        if self.line4 is None and "line4" in self.model_fields_set:
            _dict['Line4'] = None

        # set to None if postal_code (nullable) is None
        # and model_fields_set contains the field
        if self.postal_code is None and "postal_code" in self.model_fields_set:
            _dict['PostalCode'] = None

        # set to None if state_or_province (nullable) is None
        # and model_fields_set contains the field
        if self.state_or_province is None and "state_or_province" in self.model_fields_set:
            _dict['StateOrProvince'] = None

        # set to None if time_stamp (nullable) is None
        # and model_fields_set contains the field
        if self.time_stamp is None and "time_stamp" in self.model_fields_set:
            _dict['TimeStamp'] = None

        # set to None if business_name (nullable) is None
        # and model_fields_set contains the field
        if self.business_name is None and "business_name" in self.model_fields_set:
            _dict['BusinessName'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Address from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "City": obj.get("City") if obj.get("City") is not None else None,
                        "CountryCode": obj.get("CountryCode") if obj.get("CountryCode") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Line1": obj.get("Line1") if obj.get("Line1") is not None else None,
                        "Line2": obj.get("Line2") if obj.get("Line2") is not None else None,
                        "Line3": obj.get("Line3") if obj.get("Line3") is not None else None,
                        "Line4": obj.get("Line4") if obj.get("Line4") is not None else None,
                        "PostalCode": obj.get("PostalCode") if obj.get("PostalCode") is not None else None,
                        "StateOrProvince": obj.get("StateOrProvince") if obj.get("StateOrProvince") is not None else None,
                        "TimeStamp": obj.get("TimeStamp") if obj.get("TimeStamp") is not None else None,
                        "BusinessName": obj.get("BusinessName") if obj.get("BusinessName") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.recommendation_audio import RecommendationAudio
from openapi_client.models.campaign.recommendation_business_name import RecommendationBusinessName
from openapi_client.models.campaign.recommendation_logo import RecommendationLogo
from openapi_client.models.campaign.recommendation_scene import RecommendationScene
from openapi_client.models.campaign.solid_background_asset import SolidBackgroundAsset
from typing_extensions import Self

class RecommendationTemplateVideo(BaseModel):
    """
    RecommendationTemplateVideo
    """ # noqa: E501
    template_id: Optional[StrictStr] = Field(default=None, alias="templateId")
    logo: Optional[RecommendationLogo] = Field(default=None, alias="logo")
    business_name: Optional[RecommendationBusinessName] = Field(default=None, alias="businessName")
    duration: Optional[StrictInt] = Field(default=None, alias="duration")
    audio: Optional[RecommendationAudio] = Field(default=None, alias="audio")
    width: Optional[StrictInt] = Field(default=None, alias="width")
    height: Optional[StrictInt] = Field(default=None, alias="height")
    background_color: Optional[SolidBackgroundAsset] = Field(default=None, alias="backgroundColor")
    scenes: Optional[List[Optional[RecommendationScene]]] = Field(default=None, alias="scenes")
    __properties: ClassVar[List[str]] = ["templateId", "logo", "businessName", "duration", "audio", "width", "height", "backgroundColor", "scenes"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of logo
        if self.logo:
            _dict['logo'] = self.logo.to_dict()
        # override the default output from pydantic by calling `to_dict()` of business_name
        if self.business_name:
            _dict['businessName'] = self.business_name.to_dict()
        # override the default output from pydantic by calling `to_dict()` of audio
        if self.audio:
            _dict['audio'] = self.audio.to_dict()
        # override the default output from pydantic by calling `to_dict()` of background_color
        if self.background_color:
            _dict['backgroundColor'] = self.background_color.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in scenes (list)
        _items = []
        if self.scenes:
            for _item_scenes in self.scenes:
                if _item_scenes:
                    _items.append(_item_scenes.to_dict())
            _dict['Scenes'] = _items
        # set to None if template_id (nullable) is None
        # and model_fields_set contains the field
        if self.template_id is None and "template_id" in self.model_fields_set:
            _dict['templateId'] = None

        # set to None if logo (nullable) is None
        # and model_fields_set contains the field
        if self.logo is None and "logo" in self.model_fields_set:
            _dict['logo'] = None

        # set to None if business_name (nullable) is None
        # and model_fields_set contains the field
        if self.business_name is None and "business_name" in self.model_fields_set:
            _dict['businessName'] = None

        # set to None if duration (nullable) is None
        # and model_fields_set contains the field
        if self.duration is None and "duration" in self.model_fields_set:
            _dict['duration'] = None

        # set to None if audio (nullable) is None
        # and model_fields_set contains the field
        if self.audio is None and "audio" in self.model_fields_set:
            _dict['audio'] = None

        # set to None if width (nullable) is None
        # and model_fields_set contains the field
        if self.width is None and "width" in self.model_fields_set:
            _dict['width'] = None

        # set to None if height (nullable) is None
        # and model_fields_set contains the field
        if self.height is None and "height" in self.model_fields_set:
            _dict['height'] = None

        # set to None if background_color (nullable) is None
        # and model_fields_set contains the field
        if self.background_color is None and "background_color" in self.model_fields_set:
            _dict['backgroundColor'] = None

        # set to None if scenes (nullable) is None
        # and model_fields_set contains the field
        if self.scenes is None and "scenes" in self.model_fields_set:
            _dict['scenes'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RecommendationTemplateVideo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "templateId": obj.get("TemplateId") if obj.get("TemplateId") is not None else None,
                        "logo": RecommendationLogo.from_dict(obj["Logo"]) if obj.get("Logo") is not None else None,
                        "businessName": RecommendationBusinessName.from_dict(obj["BusinessName"]) if obj.get("BusinessName") is not None else None,
                        "duration": obj.get("Duration") if obj.get("Duration") is not None else None,
                        "audio": RecommendationAudio.from_dict(obj["Audio"]) if obj.get("Audio") is not None else None,
                        "width": obj.get("Width") if obj.get("Width") is not None else None,
                        "height": obj.get("Height") if obj.get("Height") is not None else None,
                        "backgroundColor": SolidBackgroundAsset.from_dict(obj["BackgroundColor"]) if obj.get("BackgroundColor") is not None else None,
                        "scenes": [RecommendationScene.from_dict(_item) for _item in obj["Scenes"]] if obj.get("Scenes") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from openapi_client.models.campaign.ad_editorial_status import AdEditorialStatus
from openapi_client.models.campaign.ad_status import AdStatus
from openapi_client.models.campaign.ad_sub_type import AdSubType
from openapi_client.models.campaign.app_install_ad import AppInstallAd
from openapi_client.models.campaign.app_url import AppUrl
from openapi_client.models.campaign.asset_link import AssetLink
from openapi_client.models.campaign.call_to_action import CallToAction
from openapi_client.models.campaign.custom_parameters import CustomParameters
from openapi_client.models.campaign.dynamic_search_ad import DynamicSearchAd
from openapi_client.models.campaign.expanded_text_ad import ExpandedTextAd
from openapi_client.models.campaign.hotel_ad import HotelAd
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.language_name import LanguageName
from openapi_client.models.campaign.product_ad import ProductAd
from openapi_client.models.campaign.responsive_ad import ResponsiveAd
from openapi_client.models.campaign.responsive_search_ad import ResponsiveSearchAd
from openapi_client.models.campaign.text_ad import TextAd
from openapi_client.models.campaign.verified_tracking_setting import VerifiedTrackingSetting
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

Ad_ONE_OF_SCHEMAS = ["AppInstallAd", "DynamicSearchAd", "ExpandedTextAd", "HotelAd", "ProductAd", "ResponsiveAd", "ResponsiveSearchAd", "TextAd"]

class Ad(BaseModel):
    """
    Ad
    """
    # data type: AppInstallAd
    oneof_schema_app_install_ad_validator: Optional[AppInstallAd] = None
    # data type: DynamicSearchAd
    oneof_schema_dynamic_search_ad_validator: Optional[DynamicSearchAd] = None
    # data type: ExpandedTextAd
    oneof_schema_expanded_text_ad_validator: Optional[ExpandedTextAd] = None
    # data type: HotelAd
    oneof_schema_hotel_ad_validator: Optional[HotelAd] = None
    # data type: ProductAd
    oneof_schema_product_ad_validator: Optional[ProductAd] = None
    # data type: ResponsiveAd
    oneof_schema_responsive_ad_validator: Optional[ResponsiveAd] = None
    # data type: ResponsiveSearchAd
    oneof_schema_responsive_search_ad_validator: Optional[ResponsiveSearchAd] = None
    # data type: TextAd
    oneof_schema_text_ad_validator: Optional[TextAd] = None
    actual_instance: Optional[Union[AppInstallAd, DynamicSearchAd, ExpandedTextAd, HotelAd, ProductAd, ResponsiveAd, ResponsiveSearchAd, TextAd]] = None
    one_of_schemas: Set[str] = { "AppInstallAd", "DynamicSearchAd", "ExpandedTextAd", "HotelAd", "ProductAd", "ResponsiveAd", "ResponsiveSearchAd", "TextAd" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        if v is None:
            return v

        instance = Ad.model_construct()
        error_messages = []
        match = 0
        # validate data type: AppInstallAd
        if not isinstance(v, AppInstallAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AppInstallAd`")
        else:
            match += 1
        # validate data type: DynamicSearchAd
        if not isinstance(v, DynamicSearchAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DynamicSearchAd`")
        else:
            match += 1
        # validate data type: ExpandedTextAd
        if not isinstance(v, ExpandedTextAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ExpandedTextAd`")
        else:
            match += 1
        # validate data type: HotelAd
        if not isinstance(v, HotelAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelAd`")
        else:
            match += 1
        # validate data type: ProductAd
        if not isinstance(v, ProductAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ProductAd`")
        else:
            match += 1
        # validate data type: ResponsiveAd
        if not isinstance(v, ResponsiveAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ResponsiveAd`")
        else:
            match += 1
        # validate data type: ResponsiveSearchAd
        if not isinstance(v, ResponsiveSearchAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ResponsiveSearchAd`")
        else:
            match += 1
        # validate data type: TextAd
        if not isinstance(v, TextAd):
            error_messages.append(f"Error! Input type `{type(v)}` is not `TextAd`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in Ad with oneOf schemas: AppInstallAd, DynamicSearchAd, ExpandedTextAd, HotelAd, ProductAd, ResponsiveAd, ResponsiveSearchAd, TextAd. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in Ad with oneOf schemas: AppInstallAd, DynamicSearchAd, ExpandedTextAd, HotelAd, ProductAd, ResponsiveAd, ResponsiveSearchAd, TextAd. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: Optional[str]) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        if json_str is None:
            return instance

        error_messages = []
        match = 0

        # use oneOf discriminator to lookup the data type
        _data_type = json.loads(json_str).get("Type")
        if not _data_type:
            raise ValueError("Failed to lookup data type from the field `Type` in the input.")

		# check if data type is `AppInstallAd`
        if _data_type == "AppInstall":
            instance.actual_instance = AppInstallAd.from_json(json_str)
            return instance
			
		# check if data type is `DynamicSearchAd`
        if _data_type == "DynamicSearch":
            instance.actual_instance = DynamicSearchAd.from_json(json_str)
            return instance
			
		# check if data type is `ExpandedTextAd`
        if _data_type == "ExpandedText":
            instance.actual_instance = ExpandedTextAd.from_json(json_str)
            return instance
			
		# check if data type is `HotelAd`
        if _data_type == "Hotel":
            instance.actual_instance = HotelAd.from_json(json_str)
            return instance
			
		# check if data type is `ProductAd`
        if _data_type == "Product":
            instance.actual_instance = ProductAd.from_json(json_str)
            return instance
			
		# check if data type is `ResponsiveAd`
        if _data_type == "ResponsiveAd":
            instance.actual_instance = ResponsiveAd.from_json(json_str)
            return instance
			
		# check if data type is `ResponsiveSearchAd`
        if _data_type == "ResponsiveSearch":
            instance.actual_instance = ResponsiveSearchAd.from_json(json_str)
            return instance
			
		# check if data type is `TextAd`
        if _data_type == "Text":
            instance.actual_instance = TextAd.from_json(json_str)
            return instance
			
		# check if data type is `AppInstallAd`
        if _data_type == "AppInstallAd":
            instance.actual_instance = AppInstallAd.from_json(json_str)
            return instance
			
		# check if data type is `DynamicSearchAd`
        if _data_type == "DynamicSearchAd":
            instance.actual_instance = DynamicSearchAd.from_json(json_str)
            return instance
			
		# check if data type is `ExpandedTextAd`
        if _data_type == "ExpandedTextAd":
            instance.actual_instance = ExpandedTextAd.from_json(json_str)
            return instance
			
		# check if data type is `HotelAd`
        if _data_type == "HotelAd":
            instance.actual_instance = HotelAd.from_json(json_str)
            return instance
			
		# check if data type is `ProductAd`
        if _data_type == "ProductAd":
            instance.actual_instance = ProductAd.from_json(json_str)
            return instance
			
		# check if data type is `ResponsiveSearchAd`
        if _data_type == "ResponsiveSearchAd":
            instance.actual_instance = ResponsiveSearchAd.from_json(json_str)
            return instance
			
		# check if data type is `TextAd`
        if _data_type == "TextAd":
            instance.actual_instance = TextAd.from_json(json_str)
            return instance
			

        # deserialize data into AppInstallAd
        try:
            instance.actual_instance = AppInstallAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DynamicSearchAd
        try:
            instance.actual_instance = DynamicSearchAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ExpandedTextAd
        try:
            instance.actual_instance = ExpandedTextAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelAd
        try:
            instance.actual_instance = HotelAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ProductAd
        try:
            instance.actual_instance = ProductAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ResponsiveAd
        try:
            instance.actual_instance = ResponsiveAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ResponsiveSearchAd
        try:
            instance.actual_instance = ResponsiveSearchAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into TextAd
        try:
            instance.actual_instance = TextAd.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into Ad with oneOf schemas: AppInstallAd, DynamicSearchAd, ExpandedTextAd, HotelAd, ProductAd, ResponsiveAd, ResponsiveSearchAd, TextAd. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into Ad with oneOf schemas: AppInstallAd, DynamicSearchAd, ExpandedTextAd, HotelAd, ProductAd, ResponsiveAd, ResponsiveSearchAd, TextAd. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], AppInstallAd, DynamicSearchAd, ExpandedTextAd, HotelAd, ProductAd, ResponsiveAd, ResponsiveSearchAd, TextAd]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

    def __getattr__(self, name):
        """Forward attribute access to actual_instance"""
        if self.actual_instance is not None and hasattr(self.actual_instance, name):
            return getattr(self.actual_instance, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, name, value):
        """Forward attribute setting to actual_instance"""
        if name in ['actual_instance', 'oneof_schema_app_install_ad_validator', 'oneof_schema_dynamic_search_ad_validator', 'oneof_schema_expanded_text_ad_validator', 'oneof_schema_hotel_ad_validator', 'oneof_schema_product_ad_validator', 'oneof_schema_responsive_ad_validator', 'oneof_schema_responsive_search_ad_validator', 'oneof_schema_text_ad_validator', 'one_of_schemas', 'model_config', 'discriminator_value_class_map']:
            super().__setattr__(name, value)
        elif self.actual_instance is not None and hasattr(self.actual_instance, name):
            setattr(self.actual_instance, name, value)
        else:
            super().__setattr__(name, value)



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class SearchCampaignChangeHistoryReportColumn(str, Enum):
    DATETIME = 'DateTime'

    ACCOUNTID = 'AccountId'

    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    CHANGEDBY = 'ChangedBy'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    ADTITLE = 'AdTitle'

    ADDESCRIPTION = 'AdDescription'

    DISPLAYURL = 'DisplayUrl'

    KEYWORD = 'Keyword'

    ITEMCHANGED = 'ItemChanged'

    ATTRIBUTECHANGED = 'AttributeChanged'

    HOWCHANGED = 'HowChanged'

    OLDVALUE = 'OldValue'

    NEWVALUE = 'NewValue'

    ENTITYNAME = 'EntityName'

    ENTITYID = 'EntityId'

    TOOL = 'Tool'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchCampaignChangeHistoryReportColumn from a JSON string"""
        return cls(json.loads(json_str))





from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from .common import _BulkAdExtensionBase
from .common import _BulkAdGroupAdExtensionAssociation
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

from bingads.v13.internal.extensions import *

_ReviewAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('ReviewAdExtension'))


class BulkReviewAdExtension(_BulkAdExtensionBase):
    """ Represents a review ad extension.

    This class exposes the :attr:`review_ad_extension` property that can be read and written
    as fields of the Review Ad Extension record in a bulk file.

    For more information, see Review Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _ReviewAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'ReviewAdExtension'
            ))
        super(BulkReviewAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )

    @property
    def review_ad_extension(self):
        """ The review ad extension.

        see Review Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @review_ad_extension.setter
    def review_ad_extension(self, value):
        self._ad_extension = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: c.review_ad_extension.Text,
            csv_to_field=lambda c, v: setattr(c.review_ad_extension, 'Text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.IsExact,
            field_to_csv=lambda c: bulk_str(c.review_ad_extension.IsExact),
            csv_to_field=lambda c, v: setattr(c.review_ad_extension, 'IsExact',
                                              v.lower() == 'true' if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Source,
            field_to_csv=lambda c: c.review_ad_extension.Source,
            csv_to_field=lambda c, v: setattr(c.review_ad_extension, 'Source', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Url,
            field_to_csv=lambda c: c.review_ad_extension.Url,
            csv_to_field=lambda c, v: setattr(c.review_ad_extension, 'Url', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.review_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('ReviewAdExtension')
        self.review_ad_extension.Type = 'ReviewAdExtension'
        super(BulkReviewAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkReviewAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.review_ad_extension, 'review_ad_extension')
        super(BulkReviewAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkReviewAdExtension._MAPPINGS)


class BulkAccountReviewAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level review ad extension.

    This class exposes properties that can be read and written
    as fields of the Account Review Ad Extension record in a bulk file.

    For more information, see Account Review Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass


class BulkCampaignReviewAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level review ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign Review Ad Extension record in a bulk file.

    For more information, see Campaign Review Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkAdGroupReviewAdExtension(_BulkAdGroupAdExtensionAssociation):
    """ Represents an ad group level Review ad extension.

    This class exposes properties that can be read and written
    as fields of the Ad Group Review Ad Extension record in a bulk file.

    For more information, see Ad Group Review Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.campaign_ad_group_ids import CampaignAdGroupIds
from openapi_client.models.campaign.frequency import Frequency
from openapi_client.models.campaign.import_option import ImportOption
from typing_extensions import Self

class GoogleImportJob(BaseModel):
    """
    GoogleImportJob
    """ # noqa: E501
    credential_id: Optional[StrictStr] = Field(default=None, alias="CredentialId")
    google_account_id: Optional[StrictStr] = Field(default=None, alias="GoogleAccountId")
    campaign_ad_group_ids: Optional[List[Optional[CampaignAdGroupIds]]] = Field(default=None, alias="CampaignAdGroupIds")
    google_user_name: Optional[StrictStr] = Field(default=None, alias="GoogleUserName")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    type: Optional[StrictStr] = Field(default='GoogleImportJob', alias="Type")
    status: Optional[StrictStr] = Field(default=None, alias="Status")
    notification_type: Optional[StrictStr] = Field(default=None, alias="NotificationType")
    created_by_user_id: Optional[StrictStr] = Field(default=None, alias="CreatedByUserId")
    created_by_user_name: Optional[StrictStr] = Field(default=None, alias="CreatedByUserName")
    frequency: Optional[Frequency] = Field(default=None, alias="Frequency")
    last_run_time_in_utc: Optional[datetime] = Field(default=None, alias="LastRunTimeInUTC")
    created_date_time_in_utc: Optional[datetime] = Field(default=None, alias="CreatedDateTimeInUTC")
    import_option: Optional[ImportOption] = Field(default=None, alias="ImportOption")
    notification_email: Optional[StrictStr] = Field(default=None, alias="NotificationEmail")
    __properties: ClassVar[List[str]] = ["CredentialId", "GoogleAccountId", "CampaignAdGroupIds", "GoogleUserName", "Name", "Id", "Type", "Status", "NotificationType", "CreatedByUserId", "CreatedByUserName", "Frequency", "LastRunTimeInUTC", "CreatedDateTimeInUTC", "ImportOption", "NotificationEmail"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GoogleImportJob from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in campaign_ad_group_ids (list)
        _items = []
        if self.campaign_ad_group_ids:
            for _item_campaign_ad_group_ids in self.campaign_ad_group_ids:
                if _item_campaign_ad_group_ids:
                    _items.append(_item_campaign_ad_group_ids.to_dict())
            _dict['CampaignAdGroupIds'] = _items
        # override the default output from pydantic by calling `to_dict()` of frequency
        if self.frequency:
            _dict['Frequency'] = self.frequency.to_dict()
        # override the default output from pydantic by calling `to_dict()` of import_option
        if self.import_option:
            _dict['ImportOption'] = self.import_option.to_dict()
        # set to None if credential_id (nullable) is None
        # and model_fields_set contains the field
        if self.credential_id is None and "credential_id" in self.model_fields_set:
            _dict['CredentialId'] = None

        # set to None if google_account_id (nullable) is None
        # and model_fields_set contains the field
        if self.google_account_id is None and "google_account_id" in self.model_fields_set:
            _dict['GoogleAccountId'] = None

        # set to None if campaign_ad_group_ids (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_ad_group_ids is None and "campaign_ad_group_ids" in self.model_fields_set:
            _dict['CampaignAdGroupIds'] = None

        # set to None if google_user_name (nullable) is None
        # and model_fields_set contains the field
        if self.google_user_name is None and "google_user_name" in self.model_fields_set:
            _dict['GoogleUserName'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if notification_type (nullable) is None
        # and model_fields_set contains the field
        if self.notification_type is None and "notification_type" in self.model_fields_set:
            _dict['NotificationType'] = None

        # set to None if created_by_user_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_user_id is None and "created_by_user_id" in self.model_fields_set:
            _dict['CreatedByUserId'] = None

        # set to None if created_by_user_name (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_user_name is None and "created_by_user_name" in self.model_fields_set:
            _dict['CreatedByUserName'] = None

        # set to None if frequency (nullable) is None
        # and model_fields_set contains the field
        if self.frequency is None and "frequency" in self.model_fields_set:
            _dict['Frequency'] = None

        # set to None if last_run_time_in_utc (nullable) is None
        # and model_fields_set contains the field
        if self.last_run_time_in_utc is None and "last_run_time_in_utc" in self.model_fields_set:
            _dict['LastRunTimeInUTC'] = None

        # set to None if created_date_time_in_utc (nullable) is None
        # and model_fields_set contains the field
        if self.created_date_time_in_utc is None and "created_date_time_in_utc" in self.model_fields_set:
            _dict['CreatedDateTimeInUTC'] = None

        # set to None if import_option (nullable) is None
        # and model_fields_set contains the field
        if self.import_option is None and "import_option" in self.model_fields_set:
            _dict['ImportOption'] = None

        # set to None if notification_email (nullable) is None
        # and model_fields_set contains the field
        if self.notification_email is None and "notification_email" in self.model_fields_set:
            _dict['NotificationEmail'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GoogleImportJob from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CredentialId": obj.get("CredentialId") if obj.get("CredentialId") is not None else None,
                        "GoogleAccountId": obj.get("GoogleAccountId") if obj.get("GoogleAccountId") is not None else None,
                        "CampaignAdGroupIds": [CampaignAdGroupIds.from_dict(_item) for _item in obj["CampaignAdGroupIds"]] if obj.get("CampaignAdGroupIds") is not None else None,
                        "GoogleUserName": obj.get("GoogleUserName") if obj.get("GoogleUserName") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'GoogleImportJob',
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "NotificationType": obj.get("NotificationType") if obj.get("NotificationType") is not None else None,
                        "CreatedByUserId": obj.get("CreatedByUserId") if obj.get("CreatedByUserId") is not None else None,
                        "CreatedByUserName": obj.get("CreatedByUserName") if obj.get("CreatedByUserName") is not None else None,
                        "Frequency": Frequency.from_dict(obj["Frequency"]) if obj.get("Frequency") is not None else None,
                        "LastRunTimeInUTC": obj.get("LastRunTimeInUTC") if obj.get("LastRunTimeInUTC") is not None else None,
                        "CreatedDateTimeInUTC": obj.get("CreatedDateTimeInUTC") if obj.get("CreatedDateTimeInUTC") is not None else None,
                        "ImportOption": ImportOption.from_dict(obj["ImportOption"]) if obj.get("ImportOption") is not None else None,
                        "NotificationEmail": obj.get("NotificationEmail") if obj.get("NotificationEmail") is not None else None
        })
        return _obj



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

class BulkBrandItem(_SingleRecordBulkEntity):
    """ Represents a brand item.

    This class exposes the property :attr:`brand_item` that can be read and written as fields of the brand item record
    in a bulk file.

    For more information, see brand item at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, id=None, brand_list_id=None, name=None, brand_name=None, brand_url=None, editorial_status=None, editorial_status_date=None, brand_item=None):
        super(BulkBrandItem, self).__init__()

        self._id = id
        self._brand_list_id = brand_list_id
        self._name = name
        self._brand_name = brand_name
        self._brand_url = brand_url
        self._editorial_status = editorial_status
        self._editorial_status_date = editorial_status_date
        self._brand_item = brand_item

    @property
    def brand_item(self):
        """ The BrandItem Data Object of the Campaign Management Service.

        A subset of BrandItem properties are available in the Brand Item record.
        For more information, see Brand Item at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._brand_item

    @brand_item.setter
    def brand_item(self, brand_item):
        self._brand_item = brand_item

    @property
    def id(self):
        """ The identifier of the brand item.

        Corresponds to the 'Id' field in the bulk file.

        :rtype: int
        """

        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @property
    def brand_list_id(self):
        """ The identifier of the brand list that contains the brand item.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._brand_list_id

    @brand_list_id.setter
    def brand_list_id(self, brand_list_id):
        self._brand_list_id = brand_list_id

    @property
    def name(self):
        """ The name of the brand item.

        Corresponds to the 'Name' field in the bulk file.

        :rtype: str
        """

        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def brand_name(self):
        """ The name of the brand.

        Corresponds to the 'Brand Name' field in the bulk file.

        :rtype: str
        """

        return self._brand_name

    @brand_name.setter
    def brand_name(self, brand_name):
        self._brand_name = brand_name

    @property
    def brand_url(self):
        """ The url of the brand.

        Corresponds to the 'Brand Url' field in the bulk file.

        :rtype: str
        """

        return self._brand_url

    @brand_url.setter
    def brand_url(self, brand_url):
        self._brand_url = brand_url

    @property
    def editorial_status(self):
        """ The editorial status

        Corresponds to the 'Editorial Status' field in the bulk file.

        :rtype: str
        """

        return self._editorial_status

    @editorial_status.setter
    def editorial_status(self, editorial_status):
        self._editorial_status = editorial_status

    @property
    def editorial_status_date(self):
        """ The editorial status date

        Corresponds to the 'Editorial Status Date' field in the bulk file.

        :rtype: str
        """

        return self._editorial_status_date

    @editorial_status_date.setter
    def editorial_status_date(self, editorial_status_date):
        self._editorial_status_date = editorial_status_date

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, 'id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: bulk_str(c.name),
            csv_to_field=lambda c, v: setattr(c, 'name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.brand_list_id),
            csv_to_field=lambda c, v: setattr(c, 'brand_list_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BrandId,
            field_to_csv=lambda c: bulk_str(c.brand_item.BrandId),
            csv_to_field=lambda c, v: setattr(c.brand_item, 'BrandId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BrandName,
            field_to_csv=lambda c: bulk_str(c.brand_name),
            csv_to_field=lambda c, v: setattr(c, 'brand_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BrandUrl,
            field_to_csv=lambda c: bulk_str(c.brand_url),
            csv_to_field=lambda c, v: setattr(c, 'brand_url', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialStatus,
            field_to_csv=lambda c: bulk_str(c.editorial_status),
            csv_to_field=lambda c, v: setattr(c, 'editorial_status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.StatusDateTime,
            field_to_csv=lambda c: bulk_datetime_str2(c.editorial_status_date),
            csv_to_field=lambda c, v: setattr(c, 'editorial_status_date', parse_datetime2(v))
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.brand_item = _CAMPAIGN_OBJECT_FACTORY_V13.create('BrandItem')

        row_values.convert_to_entity(self, BulkBrandItem._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.brand_item, 'BrandItem')
        self.convert_to_values(row_values, BulkBrandItem._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkBrandItem, self).read_additional_data(stream_reader)



__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .exceptions import *
from .enums import *
from .bulk_operation_progress_info import *
from .bulk_operation_status import *
from .download_parameters import *
from .upload_parameters import *
from .bulk_operation import *
from .file_reader import *
from .file_writer import *
from .bulk_service_manager import *
from .entities import *


# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class SuggestKeywordsFromExistingKeywordsRequest(BaseModel):
    """
    SuggestKeywordsFromExistingKeywordsRequest
    """ # noqa: E501
    keywords: Optional[List[StrictStr]] = Field(default=None, alias="Keywords")
    language: Optional[StrictStr] = Field(default=None, alias="Language")
    publisher_countries: Optional[List[StrictStr]] = Field(default=None, alias="PublisherCountries")
    max_suggestions_per_keyword: Optional[StrictInt] = Field(default=None, alias="MaxSuggestionsPerKeyword")
    suggestion_type: Optional[StrictInt] = Field(default=None, alias="SuggestionType")
    remove_duplicates: Optional[StrictBool] = Field(default=None, alias="RemoveDuplicates")
    exclude_brand: Optional[StrictBool] = Field(default=None, alias="ExcludeBrand")
    ad_group_id: Optional[StrictStr] = Field(default=None, alias="AdGroupId")
    campaign_id: Optional[StrictStr] = Field(default=None, alias="CampaignId")
    __properties: ClassVar[List[str]] = ["Keywords", "Language", "PublisherCountries", "MaxSuggestionsPerKeyword", "SuggestionType", "RemoveDuplicates", "ExcludeBrand", "AdGroupId", "CampaignId"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if keywords (nullable) is None
        # and model_fields_set contains the field
        if self.keywords is None and "keywords" in self.model_fields_set:
            _dict['Keywords'] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict['Language'] = None

        # set to None if publisher_countries (nullable) is None
        # and model_fields_set contains the field
        if self.publisher_countries is None and "publisher_countries" in self.model_fields_set:
            _dict['PublisherCountries'] = None

        # set to None if max_suggestions_per_keyword (nullable) is None
        # and model_fields_set contains the field
        if self.max_suggestions_per_keyword is None and "max_suggestions_per_keyword" in self.model_fields_set:
            _dict['MaxSuggestionsPerKeyword'] = None

        # set to None if suggestion_type (nullable) is None
        # and model_fields_set contains the field
        if self.suggestion_type is None and "suggestion_type" in self.model_fields_set:
            _dict['SuggestionType'] = None

        # set to None if remove_duplicates (nullable) is None
        # and model_fields_set contains the field
        if self.remove_duplicates is None and "remove_duplicates" in self.model_fields_set:
            _dict['RemoveDuplicates'] = None

        # set to None if exclude_brand (nullable) is None
        # and model_fields_set contains the field
        if self.exclude_brand is None and "exclude_brand" in self.model_fields_set:
            _dict['ExcludeBrand'] = None

        # set to None if ad_group_id (nullable) is None
        # and model_fields_set contains the field
        if self.ad_group_id is None and "ad_group_id" in self.model_fields_set:
            _dict['AdGroupId'] = None

        # set to None if campaign_id (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_id is None and "campaign_id" in self.model_fields_set:
            _dict['CampaignId'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SuggestKeywordsFromExistingKeywordsRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Keywords": obj.get("Keywords"),
                        "Language": obj.get("Language") if obj.get("Language") is not None else None,
                        "PublisherCountries": obj.get("PublisherCountries"),
                        "MaxSuggestionsPerKeyword": obj.get("MaxSuggestionsPerKeyword") if obj.get("MaxSuggestionsPerKeyword") is not None else None,
                        "SuggestionType": obj.get("SuggestionType") if obj.get("SuggestionType") is not None else None,
                        "RemoveDuplicates": obj.get("RemoveDuplicates") if obj.get("RemoveDuplicates") is not None else None,
                        "ExcludeBrand": obj.get("ExcludeBrand") if obj.get("ExcludeBrand") is not None else None,
                        "AdGroupId": obj.get("AdGroupId") if obj.get("AdGroupId") is not None else None,
                        "CampaignId": obj.get("CampaignId") if obj.get("CampaignId") is not None else None
        })
        return _obj



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *
from decimal import Decimal


class BulkNewCustomerAcquisitionGoal (_SingleRecordBulkEntity):
    """ Represents a new customer acquisition goal that can be read or written in a bulk file.

    Properties of this class and of classes that it is derived from, correspond to fields of the Budget record in a bulk file.
    For more information, see Budget at https://go.microsoft.com/fwlink/?linkid=846127

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, new_customer_acquisition_goal=None, target=None):
        super(BulkNewCustomerAcquisitionGoal , self).__init__()
        self._new_customer_acquisition_goal = new_customer_acquisition_goal
        self._target = target

    @property
    def new_customer_acquisition_goal (self):
        """
        the NewCustomerAcquisitionGoal object, see more detail at: https://go.microsoft.com/fwlink/?linkid=846127
        """
        return self._new_customer_acquisition_goal

    @new_customer_acquisition_goal .setter
    def new_customer_acquisition_goal (self, value):
        self._new_customer_acquisition_goal  = value

    @property
    def target(self):
        """
        The ids of audiences within the new customer acquisition.
        It should be split by simicolon. example: "123;456;789"
        Corresponds to 'Target' field in bulk file.
        :rtype: str
        """
        return self._target

    @target.setter
    def target(self, value):
        self._target = value


    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.new_customer_acquisition_goal .Id),
            csv_to_field=lambda c, v: setattr(c.new_customer_acquisition_goal , 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Target,
            field_to_csv=lambda c: bulk_str(c.target),
            csv_to_field=lambda c, v: setattr(c, 'target', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdditionalConversionValue,
            field_to_csv=lambda c: bulk_str(c.new_customer_acquisition_goal.AdditionalValue),
            csv_to_field=lambda c, v: setattr(c.new_customer_acquisition_goal , 'AdditionalValue', Decimal(v) if v else None)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self._new_customer_acquisition_goal  = _CAMPAIGN_OBJECT_FACTORY_V13.create('NewCustomerAcquisitionGoal')
        row_values.convert_to_entity(self, BulkNewCustomerAcquisitionGoal ._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.new_customer_acquisition_goal , 'new_customer_acquisition_goal ')
        self.convert_to_values(row_values, BulkNewCustomerAcquisitionGoal ._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkNewCustomerAcquisitionGoal , self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class FaultSource(str, Enum):
    MIDDLETIER = 'MiddleTier'

    DATABASE = 'Database'

    EDITORIAL = 'Editorial'

    AUTHENTICATION = 'Authentication'

    AUTHORIZATION = 'Authorization'

    PUBLISHER = 'Publisher'

    AUDIENCEINTELIGENCE = 'AudienceInteligence'

    CAM = 'Cam'

    DAM = 'Dam'

    FLEXTAG = 'FlexTag'

    BINGMERCHANTCENTER = 'BingMerchantCenter'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FaultSource from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AppsPerformanceReportColumn(str, Enum):
    TIMEPERIOD = 'TimePeriod'

    ACCOUNTID = 'AccountId'

    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNID = 'CampaignId'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ASSETGROUPSTATUS = 'AssetGroupStatus'

    CURRENCYCODE = 'CurrencyCode'

    LANGUAGE = 'Language'

    BIDSTRATEGYTYPE = 'BidStrategyType'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    AVERAGECPC = 'AverageCpc'

    SPEND = 'Spend'

    AVERAGECPM = 'AverageCpm'

    ALLCONVERSIONS = 'AllConversions'

    ALLCOSTPERCONVERSION = 'AllCostPerConversion'

    ALLCONVERSIONRATE = 'AllConversionRate'

    CONVERSIONS = 'Conversions'

    CONVERSIONRATE = 'ConversionRate'

    COSTPERCONVERSION = 'CostPerConversion'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    VIEWTHROUGHCONVERSIONRATE = 'ViewThroughConversionRate'

    VIEWTHROUGHCOSTPERCONVERSION = 'ViewThroughCostPerConversion'

    DOWNLOADS = 'Downloads'

    POSTCLICKDOWNLOADRATE = 'PostClickDownloadRate'

    COSTPERDOWNLOAD = 'CostPerDownload'

    APPINSTALLS = 'AppInstalls'

    POSTCLICKINSTALLRATE = 'PostClickInstallRate'

    CPI = 'CPI'

    PURCHASES = 'Purchases'

    POSTINSTALLPURCHASERATE = 'PostInstallPurchaseRate'

    CPP = 'CPP'

    SUBSCRIPTIONS = 'Subscriptions'

    POSTINSTALLSUBSCRIPTIONRATE = 'PostInstallSubscriptionRate'

    CPS = 'CPS'

    REVENUE = 'Revenue'

    RETURNONADSPEND = 'ReturnOnAdSpend'

    REVENUEPERDOWNLOAD = 'RevenuePerDownload'

    REVENUEPERAPPINSTALL = 'RevenuePerAppInstall'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AppsPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *

class BulkAssetGroupUrlTarget(_SingleRecordBulkEntity):
    """ Represents an asset group url target.

    This class exposes the property :attr:`asset_group_url_target` that can be read and written as fields of the asset group url target record
    in a bulk file.

    For more information, see Asset Group at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None):
        super(BulkAssetGroupUrlTarget, self).__init__()

        self._status = status
        self._id = None
        self._asset_group_id = None
        self._target_condition1 = None
        self._target_condition2 = None
        self._target_condition3 = None
        self._target_condition_operator1 = None
        self._target_condition_operator2 = None
        self._target_condition_operator3 = None
        self._target_value1 = None
        self._target_value2 = None
        self._target_value3 = None

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @property
    def asset_group_id(self):
        return self._asset_group_id

    @asset_group_id.setter
    def asset_group_id(self, asset_group_id):
        self._asset_group_id = asset_group_id

    @property
    def target_condition1(self):
        return self._target_condition1

    @target_condition1.setter
    def target_condition1(self, target_condition1):
        self._target_condition1 = target_condition1

    @property
    def target_condition2(self):
        return self._target_condition2

    @target_condition2.setter
    def target_condition2(self, target_condition2):
        self._target_condition2 = target_condition2

    @property
    def target_condition3(self):
        return self._target_condition3

    @target_condition3.setter
    def target_condition3(self, target_condition3):
        self._target_condition3 = target_condition3

    @property
    def target_condition_operator1(self):
        return self._target_condition_operator1

    @target_condition_operator1.setter
    def target_condition_operator1(self, target_condition_operator1):
        self._target_condition_operator1 = target_condition_operator1

    @property
    def target_condition_operator2(self):
        return self._target_condition_operator2

    @target_condition_operator2.setter
    def target_condition_operator2(self, target_condition_operator2):
        self._target_condition_operator2 = target_condition_operator2

    @property
    def target_condition_operator3(self):
        return self._target_condition_operator3

    @target_condition_operator3.setter
    def target_condition_operator3(self, target_condition_operator3):
        self._target_condition_operator3 = target_condition_operator3

    @property
    def target_value1(self):
        return self._target_value1

    @target_value1.setter
    def target_value1(self, target_value1):
        self._target_value1 = target_value1

    @property
    def target_value2(self):
        return self._target_value2

    @target_value2.setter
    def target_value2(self, target_value2):
        self._target_value2 = target_value2

    @property
    def target_value3(self):
        return self._target_value3

    @target_value3.setter
    def target_value3(self, target_value3):
        self._target_value3 = target_value3

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, 'id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.asset_group_id),
            csv_to_field=lambda c, v: setattr(c, 'asset_group_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetCondition1,
            field_to_csv=lambda c: c.target_condition1,
            csv_to_field=lambda c, v: setattr(c, 'target_condition1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetCondition2,
            field_to_csv=lambda c: c.target_condition2,
            csv_to_field=lambda c, v: setattr(c, 'target_condition2', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetCondition3,
            field_to_csv=lambda c: c.target_condition3,
            csv_to_field=lambda c, v: setattr(c, 'target_condition3', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetConditionOperator1,
            field_to_csv=lambda c: c.target_condition_operator1,
            csv_to_field=lambda c, v: setattr(c, 'target_condition_operator1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetConditionOperator2,
            field_to_csv=lambda c: c.target_condition_operator2,
            csv_to_field=lambda c, v: setattr(c, 'target_condition_operator2', v)
        ),_SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetConditionOperator3,
            field_to_csv=lambda c: c.target_condition_operator3,
            csv_to_field=lambda c, v: setattr(c, 'target_condition_operator3', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetValue1,
            field_to_csv=lambda c: c.target_value1,
            csv_to_field=lambda c, v: setattr(c, 'target_value1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetValue2,
            field_to_csv=lambda c: c.target_value2,
            csv_to_field=lambda c, v: setattr(c, 'target_value2', v)
        ),_SimpleBulkMapping(
            header=_StringTable.AssetGroupTargetValue3,
            field_to_csv=lambda c: c.target_value3,
            csv_to_field=lambda c, v: setattr(c, 'target_value3', v)
        ),
    ]


    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkAssetGroupUrlTarget._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkAssetGroupUrlTarget._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAssetGroupUrlTarget, self).read_additional_data(stream_reader)



# flake8: noqa

# import apis into api package
from openapi_client.api.bulk_service_api import BulkServiceApi
from openapi_client.api.campaign_management_service_api import CampaignManagementServiceApi
from openapi_client.api.reporting_service_api import ReportingServiceApi
from openapi_client.api.customer_management_service_api import CustomerManagementServiceApi
from openapi_client.api.customer_billing_service_api import CustomerBillingServiceApi
from openapi_client.api.ad_insight_service_api import AdInsightServiceApi



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *
from decimal import Decimal


class BulkCustomerListItem(_SingleRecordBulkEntity):
    """ Represents a customer list item that can be read or written in a bulk file.

    Properties of this class and of classes that it is derived from, correspond to fields of the Customer List Item record in a bulk file.
    For more information, see Budget at https://go.microsoft.com/fwlink/?linkid=846127

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, audience=None, parent_id=None, sub_type=None, text = None):
        super(BulkCustomerListItem, self).__init__()
        self._audience = audience
        self._parent_id = parent_id
        self._sub_type = sub_type
        self._text = text

    @property
    def audience(self):
        """
        the audience, see more detail at: https://go.microsoft.com/fwlink/?linkid=846127
        
        :rtype: str
        """
        return self._audience

    @audience.setter
    def audience(self, value):
        self._audience = value

    @property
    def parent_id(self):
        """ the parent id of bulk record
        Corresponds to the 'parent id' field in the bulk file.

        :rtype: long
        """
        return self._parent_id

    @parent_id.setter
    def parent_id(self, value):
        self._parent_id = value

    @property
    def sub_type(self):
        """ the sub type of bulk record
        Corresponds to the 'Sub Type' field in the bulk file.

        :rtype: str
        """
        return self._sub_type

    @sub_type.setter
    def sub_type(self, value):
        self._sub_type = value
        
    @property
    def text(self):
        """ the text of bulk record
        Corresponds to the 'Text' field in the bulk file.

        :rtype: str
        """
        return self._text

    @text.setter
    def text(self, value):
        self._text = value


    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.parent_id),
            csv_to_field=lambda c, v: setattr(c, 'parent_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Audience,
            field_to_csv=lambda c: bulk_str(c.audience),
            csv_to_field=lambda c, v: setattr(c, 'audience', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SubType,
            field_to_csv=lambda c: c.sub_type,
            csv_to_field=lambda c, v: setattr(c, 'sub_type', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: bulk_str(c.text),
            csv_to_field=lambda c, v: setattr(c, 'text', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkCustomerListItem._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkCustomerListItem._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkCustomerListItem, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from openapi_client.models.campaign.app_setting import AppSetting
from openapi_client.models.campaign.app_store import AppStore
from openapi_client.models.campaign.bid_option import BidOption
from openapi_client.models.campaign.call_to_action_setting import CallToActionSetting
from openapi_client.models.campaign.co_op_setting import CoOpSetting
from openapi_client.models.campaign.disclaimer_setting import DisclaimerSetting
from openapi_client.models.campaign.dynamic_feed_setting import DynamicFeedSetting
from openapi_client.models.campaign.dynamic_search_ads_setting import DynamicSearchAdsSetting
from openapi_client.models.campaign.dynamic_search_ads_source import DynamicSearchAdsSource
from openapi_client.models.campaign.hotel_ad_group_type import HotelAdGroupType
from openapi_client.models.campaign.hotel_setting import HotelSetting
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.new_customer_acquisition_goal_setting import NewCustomerAcquisitionGoalSetting
from openapi_client.models.campaign.performance_max_setting import PerformanceMaxSetting
from openapi_client.models.campaign.responsive_search_ads_setting import ResponsiveSearchAdsSetting
from openapi_client.models.campaign.shopping_setting import ShoppingSetting
from openapi_client.models.campaign.target_setting import TargetSetting
from openapi_client.models.campaign.third_party_measurement_setting import ThirdPartyMeasurementSetting
from openapi_client.models.campaign.vanity_pharma_display_url_mode import VanityPharmaDisplayUrlMode
from openapi_client.models.campaign.vanity_pharma_setting import VanityPharmaSetting
from openapi_client.models.campaign.vanity_pharma_website_description import VanityPharmaWebsiteDescription
from openapi_client.models.campaign.verified_tracking_setting import VerifiedTrackingSetting
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

Setting_ONE_OF_SCHEMAS = ["AppSetting", "CallToActionSetting", "CoOpSetting", "DisclaimerSetting", "DynamicFeedSetting", "DynamicSearchAdsSetting", "HotelSetting", "NewCustomerAcquisitionGoalSetting", "PerformanceMaxSetting", "ResponsiveSearchAdsSetting", "ShoppingSetting", "TargetSetting", "ThirdPartyMeasurementSetting", "VanityPharmaSetting", "VerifiedTrackingSetting"]

class Setting(BaseModel):
    """
    Setting
    """
    # data type: AppSetting
    oneof_schema_app_setting_validator: Optional[AppSetting] = None
    # data type: CallToActionSetting
    oneof_schema_call_to_action_setting_validator: Optional[CallToActionSetting] = None
    # data type: CoOpSetting
    oneof_schema_co_op_setting_validator: Optional[CoOpSetting] = None
    # data type: DisclaimerSetting
    oneof_schema_disclaimer_setting_validator: Optional[DisclaimerSetting] = None
    # data type: DynamicFeedSetting
    oneof_schema_dynamic_feed_setting_validator: Optional[DynamicFeedSetting] = None
    # data type: DynamicSearchAdsSetting
    oneof_schema_dynamic_search_ads_setting_validator: Optional[DynamicSearchAdsSetting] = None
    # data type: HotelSetting
    oneof_schema_hotel_setting_validator: Optional[HotelSetting] = None
    # data type: NewCustomerAcquisitionGoalSetting
    oneof_schema_new_customer_acquisition_goal_setting_validator: Optional[NewCustomerAcquisitionGoalSetting] = None
    # data type: PerformanceMaxSetting
    oneof_schema_performance_max_setting_validator: Optional[PerformanceMaxSetting] = None
    # data type: ResponsiveSearchAdsSetting
    oneof_schema_responsive_search_ads_setting_validator: Optional[ResponsiveSearchAdsSetting] = None
    # data type: ShoppingSetting
    oneof_schema_shopping_setting_validator: Optional[ShoppingSetting] = None
    # data type: TargetSetting
    oneof_schema_target_setting_validator: Optional[TargetSetting] = None
    # data type: ThirdPartyMeasurementSetting
    oneof_schema_third_party_measurement_setting_validator: Optional[ThirdPartyMeasurementSetting] = None
    # data type: VanityPharmaSetting
    oneof_schema_vanity_pharma_setting_validator: Optional[VanityPharmaSetting] = None
    # data type: VerifiedTrackingSetting
    oneof_schema_verified_tracking_setting_validator: Optional[VerifiedTrackingSetting] = None
    actual_instance: Optional[Union[AppSetting, CallToActionSetting, CoOpSetting, DisclaimerSetting, DynamicFeedSetting, DynamicSearchAdsSetting, HotelSetting, NewCustomerAcquisitionGoalSetting, PerformanceMaxSetting, ResponsiveSearchAdsSetting, ShoppingSetting, TargetSetting, ThirdPartyMeasurementSetting, VanityPharmaSetting, VerifiedTrackingSetting]] = None
    one_of_schemas: Set[str] = { "AppSetting", "CallToActionSetting", "CoOpSetting", "DisclaimerSetting", "DynamicFeedSetting", "DynamicSearchAdsSetting", "HotelSetting", "NewCustomerAcquisitionGoalSetting", "PerformanceMaxSetting", "ResponsiveSearchAdsSetting", "ShoppingSetting", "TargetSetting", "ThirdPartyMeasurementSetting", "VanityPharmaSetting", "VerifiedTrackingSetting" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        if v is None:
            return v

        instance = Setting.model_construct()
        error_messages = []
        match = 0
        # validate data type: AppSetting
        if not isinstance(v, AppSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AppSetting`")
        else:
            match += 1
        # validate data type: CallToActionSetting
        if not isinstance(v, CallToActionSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CallToActionSetting`")
        else:
            match += 1
        # validate data type: CoOpSetting
        if not isinstance(v, CoOpSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CoOpSetting`")
        else:
            match += 1
        # validate data type: DisclaimerSetting
        if not isinstance(v, DisclaimerSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DisclaimerSetting`")
        else:
            match += 1
        # validate data type: DynamicFeedSetting
        if not isinstance(v, DynamicFeedSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DynamicFeedSetting`")
        else:
            match += 1
        # validate data type: DynamicSearchAdsSetting
        if not isinstance(v, DynamicSearchAdsSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DynamicSearchAdsSetting`")
        else:
            match += 1
        # validate data type: HotelSetting
        if not isinstance(v, HotelSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelSetting`")
        else:
            match += 1
        # validate data type: NewCustomerAcquisitionGoalSetting
        if not isinstance(v, NewCustomerAcquisitionGoalSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `NewCustomerAcquisitionGoalSetting`")
        else:
            match += 1
        # validate data type: PerformanceMaxSetting
        if not isinstance(v, PerformanceMaxSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PerformanceMaxSetting`")
        else:
            match += 1
        # validate data type: ResponsiveSearchAdsSetting
        if not isinstance(v, ResponsiveSearchAdsSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ResponsiveSearchAdsSetting`")
        else:
            match += 1
        # validate data type: ShoppingSetting
        if not isinstance(v, ShoppingSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ShoppingSetting`")
        else:
            match += 1
        # validate data type: TargetSetting
        if not isinstance(v, TargetSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `TargetSetting`")
        else:
            match += 1
        # validate data type: ThirdPartyMeasurementSetting
        if not isinstance(v, ThirdPartyMeasurementSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ThirdPartyMeasurementSetting`")
        else:
            match += 1
        # validate data type: VanityPharmaSetting
        if not isinstance(v, VanityPharmaSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `VanityPharmaSetting`")
        else:
            match += 1
        # validate data type: VerifiedTrackingSetting
        if not isinstance(v, VerifiedTrackingSetting):
            error_messages.append(f"Error! Input type `{type(v)}` is not `VerifiedTrackingSetting`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in Setting with oneOf schemas: AppSetting, CallToActionSetting, CoOpSetting, DisclaimerSetting, DynamicFeedSetting, DynamicSearchAdsSetting, HotelSetting, NewCustomerAcquisitionGoalSetting, PerformanceMaxSetting, ResponsiveSearchAdsSetting, ShoppingSetting, TargetSetting, ThirdPartyMeasurementSetting, VanityPharmaSetting, VerifiedTrackingSetting. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in Setting with oneOf schemas: AppSetting, CallToActionSetting, CoOpSetting, DisclaimerSetting, DynamicFeedSetting, DynamicSearchAdsSetting, HotelSetting, NewCustomerAcquisitionGoalSetting, PerformanceMaxSetting, ResponsiveSearchAdsSetting, ShoppingSetting, TargetSetting, ThirdPartyMeasurementSetting, VanityPharmaSetting, VerifiedTrackingSetting. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: Optional[str]) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        if json_str is None:
            return instance

        error_messages = []
        match = 0

        # use oneOf discriminator to lookup the data type
        _data_type = json.loads(json_str).get("Type")
        if not _data_type:
            raise ValueError("Failed to lookup data type from the field `Type` in the input.")

		# check if data type is `AppSetting`
        if _data_type == "AppSetting":
            instance.actual_instance = AppSetting.from_json(json_str)
            return instance
			
		# check if data type is `CallToActionSetting`
        if _data_type == "CallToActionSetting":
            instance.actual_instance = CallToActionSetting.from_json(json_str)
            return instance
			
		# check if data type is `CoOpSetting`
        if _data_type == "CoOpSetting":
            instance.actual_instance = CoOpSetting.from_json(json_str)
            return instance
			
		# check if data type is `DisclaimerSetting`
        if _data_type == "DisclaimerSetting":
            instance.actual_instance = DisclaimerSetting.from_json(json_str)
            return instance
			
		# check if data type is `DynamicFeedSetting`
        if _data_type == "DynamicFeedSetting":
            instance.actual_instance = DynamicFeedSetting.from_json(json_str)
            return instance
			
		# check if data type is `DynamicSearchAdsSetting`
        if _data_type == "DynamicSearchAdsSetting":
            instance.actual_instance = DynamicSearchAdsSetting.from_json(json_str)
            return instance
			
		# check if data type is `HotelSetting`
        if _data_type == "HotelSetting":
            instance.actual_instance = HotelSetting.from_json(json_str)
            return instance
			
		# check if data type is `NewCustomerAcquisitionGoalSetting`
        if _data_type == "NewCustomerAcquisitionGoalSetting":
            instance.actual_instance = NewCustomerAcquisitionGoalSetting.from_json(json_str)
            return instance
			
		# check if data type is `PerformanceMaxSetting`
        if _data_type == "PerformanceMaxSetting":
            instance.actual_instance = PerformanceMaxSetting.from_json(json_str)
            return instance
			
		# check if data type is `ResponsiveSearchAdsSetting`
        if _data_type == "ResponsiveSearchAdsSetting":
            instance.actual_instance = ResponsiveSearchAdsSetting.from_json(json_str)
            return instance
			
		# check if data type is `ShoppingSetting`
        if _data_type == "ShoppingSetting":
            instance.actual_instance = ShoppingSetting.from_json(json_str)
            return instance
			
		# check if data type is `TargetSetting`
        if _data_type == "TargetSetting":
            instance.actual_instance = TargetSetting.from_json(json_str)
            return instance
			
		# check if data type is `ThirdPartyMeasurementSetting`
        if _data_type == "ThirdPartyMeasurementSetting":
            instance.actual_instance = ThirdPartyMeasurementSetting.from_json(json_str)
            return instance
			
		# check if data type is `VanityPharmaSetting`
        if _data_type == "VanityPharmaSetting":
            instance.actual_instance = VanityPharmaSetting.from_json(json_str)
            return instance
			
		# check if data type is `VerifiedTrackingSetting`
        if _data_type == "VerifiedTrackingSetting":
            instance.actual_instance = VerifiedTrackingSetting.from_json(json_str)
            return instance
			

        # deserialize data into AppSetting
        try:
            instance.actual_instance = AppSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CallToActionSetting
        try:
            instance.actual_instance = CallToActionSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CoOpSetting
        try:
            instance.actual_instance = CoOpSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DisclaimerSetting
        try:
            instance.actual_instance = DisclaimerSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DynamicFeedSetting
        try:
            instance.actual_instance = DynamicFeedSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DynamicSearchAdsSetting
        try:
            instance.actual_instance = DynamicSearchAdsSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelSetting
        try:
            instance.actual_instance = HotelSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into NewCustomerAcquisitionGoalSetting
        try:
            instance.actual_instance = NewCustomerAcquisitionGoalSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PerformanceMaxSetting
        try:
            instance.actual_instance = PerformanceMaxSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ResponsiveSearchAdsSetting
        try:
            instance.actual_instance = ResponsiveSearchAdsSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ShoppingSetting
        try:
            instance.actual_instance = ShoppingSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into TargetSetting
        try:
            instance.actual_instance = TargetSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ThirdPartyMeasurementSetting
        try:
            instance.actual_instance = ThirdPartyMeasurementSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into VanityPharmaSetting
        try:
            instance.actual_instance = VanityPharmaSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into VerifiedTrackingSetting
        try:
            instance.actual_instance = VerifiedTrackingSetting.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into Setting with oneOf schemas: AppSetting, CallToActionSetting, CoOpSetting, DisclaimerSetting, DynamicFeedSetting, DynamicSearchAdsSetting, HotelSetting, NewCustomerAcquisitionGoalSetting, PerformanceMaxSetting, ResponsiveSearchAdsSetting, ShoppingSetting, TargetSetting, ThirdPartyMeasurementSetting, VanityPharmaSetting, VerifiedTrackingSetting. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into Setting with oneOf schemas: AppSetting, CallToActionSetting, CoOpSetting, DisclaimerSetting, DynamicFeedSetting, DynamicSearchAdsSetting, HotelSetting, NewCustomerAcquisitionGoalSetting, PerformanceMaxSetting, ResponsiveSearchAdsSetting, ShoppingSetting, TargetSetting, ThirdPartyMeasurementSetting, VanityPharmaSetting, VerifiedTrackingSetting. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], AppSetting, CallToActionSetting, CoOpSetting, DisclaimerSetting, DynamicFeedSetting, DynamicSearchAdsSetting, HotelSetting, NewCustomerAcquisitionGoalSetting, PerformanceMaxSetting, ResponsiveSearchAdsSetting, ShoppingSetting, TargetSetting, ThirdPartyMeasurementSetting, VanityPharmaSetting, VerifiedTrackingSetting]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

    def __getattr__(self, name):
        """Forward attribute access to actual_instance"""
        if self.actual_instance is not None and hasattr(self.actual_instance, name):
            return getattr(self.actual_instance, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, name, value):
        """Forward attribute setting to actual_instance"""
        if name in ['actual_instance', 'oneof_schema_app_setting_validator', 'oneof_schema_call_to_action_setting_validator', 'oneof_schema_co_op_setting_validator', 'oneof_schema_disclaimer_setting_validator', 'oneof_schema_dynamic_feed_setting_validator', 'oneof_schema_dynamic_search_ads_setting_validator', 'oneof_schema_hotel_setting_validator', 'oneof_schema_new_customer_acquisition_goal_setting_validator', 'oneof_schema_performance_max_setting_validator', 'oneof_schema_responsive_search_ads_setting_validator', 'oneof_schema_shopping_setting_validator', 'oneof_schema_target_setting_validator', 'oneof_schema_third_party_measurement_setting_validator', 'oneof_schema_vanity_pharma_setting_validator', 'oneof_schema_verified_tracking_setting_validator', 'one_of_schemas', 'model_config', 'discriminator_value_class_map']:
            super().__setattr__(name, value)
        elif self.actual_instance is not None and hasattr(self.actual_instance, name):
            setattr(self.actual_instance, name, value)
        else:
            super().__setattr__(name, value)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.bmc_store_sub_type import BMCStoreSubType
from typing_extensions import Self

class BMCStore(BaseModel):
    """
    BMCStore
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    is_active: Optional[StrictBool] = Field(default=None, alias="IsActive")
    is_product_ads_enabled: Optional[StrictBool] = Field(default=None, alias="IsProductAdsEnabled")
    has_catalog: Optional[StrictBool] = Field(default=None, alias="HasCatalog")
    sub_type: Optional[BMCStoreSubType] = Field(default=None, alias="SubType")
    store_url: Optional[StrictStr] = Field(default=None, alias="StoreUrl")
    __properties: ClassVar[List[str]] = ["Id", "Name", "IsActive", "IsProductAdsEnabled", "HasCatalog", "SubType", "StoreUrl"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if is_active (nullable) is None
        # and model_fields_set contains the field
        if self.is_active is None and "is_active" in self.model_fields_set:
            _dict['IsActive'] = None

        # set to None if is_product_ads_enabled (nullable) is None
        # and model_fields_set contains the field
        if self.is_product_ads_enabled is None and "is_product_ads_enabled" in self.model_fields_set:
            _dict['IsProductAdsEnabled'] = None

        # set to None if has_catalog (nullable) is None
        # and model_fields_set contains the field
        if self.has_catalog is None and "has_catalog" in self.model_fields_set:
            _dict['HasCatalog'] = None

        # set to None if sub_type (nullable) is None
        # and model_fields_set contains the field
        if self.sub_type is None and "sub_type" in self.model_fields_set:
            _dict['SubType'] = None

        # set to None if store_url (nullable) is None
        # and model_fields_set contains the field
        if self.store_url is None and "store_url" in self.model_fields_set:
            _dict['StoreUrl'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BMCStore from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "IsActive": obj.get("IsActive") if obj.get("IsActive") is not None else None,
                        "IsProductAdsEnabled": obj.get("IsProductAdsEnabled") if obj.get("IsProductAdsEnabled") is not None else None,
                        "HasCatalog": obj.get("HasCatalog") if obj.get("HasCatalog") is not None else None,
                        "SubType": obj.get("SubType") if obj.get("SubType") is not None else None,
                        "StoreUrl": obj.get("StoreUrl") if obj.get("StoreUrl") is not None else None
        })
        return _obj



from .string_table import _StringTable


class _CsvHeaders:
    HEADERS = [
        # Common
        _StringTable.Type,
        _StringTable.Status,
        _StringTable.Id,
        _StringTable.ParentId,
        _StringTable.CampaignId,
        _StringTable.SubType,
        _StringTable.Campaign,
        _StringTable.AdGroup,
        _StringTable.Website,
        _StringTable.SyncTime,
        _StringTable.ClientId,
        _StringTable.LastModifiedTime,
        _StringTable.MultiMediaAdBidAdjustment,
        _StringTable.UseOptimizedTargeting,
        _StringTable.DynamicDescriptionEnabled,
        _StringTable.Details,


        # Campaign
        _StringTable.TimeZone,
        _StringTable.Budget,
        _StringTable.BudgetType,
        _StringTable.BudgetName,
        _StringTable.BudgetId,
        _StringTable.DestinationChannel,
        _StringTable.IsMultiChannelCampaign,

        # AdGroup
        _StringTable.StartDate,
        _StringTable.EndDate,
        _StringTable.NetworkDistribution,
        _StringTable.AdRotation,
        _StringTable.CpcBid,
        _StringTable.CpvBid,
        _StringTable.CpmBid,
        _StringTable.Language,
        _StringTable.PrivacyStatus,
        _StringTable.AdGroupType,
        _StringTable.HotelAdGroupType,
        _StringTable.CommissionRate,
        _StringTable.PercentCpcBid,
        _StringTable.FinalUrlExpansionOptOut,
        _StringTable.HotelListingGroupType,
        _StringTable.HotelAttribute,
        _StringTable.HotelAttributeValue,

        # OnlineConversionAdjustment
        _StringTable.TransactionId,
        # Ads
        _StringTable.Title,
        _StringTable.Text,
        _StringTable.TextPart2,
        _StringTable.DisplayUrl,
        _StringTable.DestinationUrl,
        _StringTable.BusinessName,
        _StringTable.PhoneNumber,
        _StringTable.PromotionalText,
        _StringTable.EditorialStatus,
        _StringTable.EditorialLocation,
        _StringTable.EditorialTerm,
        _StringTable.EditorialReasonCode,
        _StringTable.EditorialAppealStatus,
        _StringTable.DevicePreference,

        # Keywords
        _StringTable.Keyword,
        _StringTable.MatchType,
        _StringTable.Bid,
        _StringTable.Param1,
        _StringTable.Param2,
        _StringTable.Param3,

        # Location Target
        _StringTable.Target,
        _StringTable.TargetAll,
        _StringTable.BidAdjustment,
        _StringTable.CashbackAdjustment,
        _StringTable.RadiusTargetId,
        _StringTable.Name,
        _StringTable.OsNames,
        _StringTable.Radius,
        _StringTable.Unit,
        _StringTable.BusinessId,

        # DayTime Target
        _StringTable.FromHour,
        _StringTable.FromMinute,
        _StringTable.ToHour,
        _StringTable.ToMinute,

        # Profile Criterion
        _StringTable.Profile,
        _StringTable.ProfileId,

        # AdExtensions common
        _StringTable.Version,

        #Disclaimer Ad Extension
        _StringTable.DisclaimerAdsEnabled,
        _StringTable.DisclaimerName,
        _StringTable.DisclaimerTitle,
        _StringTable.DisclaimerLayout,
        _StringTable.DisclaimerPopupText,
        _StringTable.DisclaimerLineText,

        # SiteLink Ad Extensions
        _StringTable.SiteLinkExtensionOrder,
        _StringTable.SiteLinkDisplayText,
        _StringTable.SiteLinkDestinationUrl,
        _StringTable.SiteLinkDescription1,
        _StringTable.SiteLinkDescription2,

        # Location Ad Extensions
        _StringTable.GeoCodeStatus,
        _StringTable.ImageMediaId,
        _StringTable.AddressLine1,
        _StringTable.AddressLine2,
        _StringTable.PostalCode,
        _StringTable.City,
        _StringTable.StateOrProvince,
        _StringTable.ProvinceName,
        _StringTable.Latitude,
        _StringTable.Longitude,

        # Call Ad Extensions
        _StringTable.CountryCode,
        _StringTable.IsCallOnly,
        _StringTable.IsCallTrackingEnabled,
        _StringTable.RequireTollFreeTrackingNumber,

        # Structured Snippet Ad Extensions
        _StringTable.StructuredSnippetHeader,
        _StringTable.StructuredSnippetValues,

        # Image Ad Extensions
        _StringTable.AltText,
        _StringTable.MediaIds,
        _StringTable.PublisherCountries,
        _StringTable.Layouts,
        _StringTable.DisplayText,

        # Filter link ad extension
        _StringTable.AdExtensionHeaderType,
        _StringTable.Texts,

        # Image
        _StringTable.Height,
        _StringTable.Width,

        # Video
        _StringTable.SourceUrl,
        _StringTable.AspectRatio,
        _StringTable.DurationInMillionSeconds,

        # Callout Ad Extension
        _StringTable.CalloutText,

        #Flyer Ad Extension
        _StringTable.FlyerAdExtension,
        _StringTable.AccountFlyerAdExtension,
        _StringTable.CampaignFlyerAdExtension,
        _StringTable.AdGroupFlyerAdExtension,
        _StringTable.FlyerName,
        _StringTable.MediaUrls,

        #Video Ad Extension
        _StringTable.ThumbnailUrl,
        _StringTable.ThumbnailId,
        _StringTable.VideoId,

        # Product Target
        _StringTable.MerchantCenterId,
        _StringTable.MerchantCenterName,
        _StringTable.ProductCondition1,
        _StringTable.ProductValue1,
        _StringTable.ProductCondition2,
        _StringTable.ProductValue2,
        _StringTable.ProductCondition3,
        _StringTable.ProductValue3,
        _StringTable.ProductCondition4,
        _StringTable.ProductValue4,
        _StringTable.ProductCondition5,
        _StringTable.ProductValue5,
        _StringTable.ProductCondition6,
        _StringTable.ProductValue6,
        _StringTable.ProductCondition7,
        _StringTable.ProductValue7,
        _StringTable.ProductCondition8,
        _StringTable.ProductValue8,

        # BI
        _StringTable.Spend,
        _StringTable.Impressions,
        _StringTable.Clicks,
        _StringTable.CTR,
        _StringTable.AvgCPC,
        _StringTable.AvgCPM,
        _StringTable.AvgPosition,
        _StringTable.Conversions,
        _StringTable.CPA,

        _StringTable.QualityScore,
        _StringTable.KeywordRelevance,
        _StringTable.LandingPageRelevance,
        _StringTable.LandingPageUserExperience,

        _StringTable.AppPlatform,
        _StringTable.AppStoreId,
        _StringTable.IsTrackingEnabled,

        _StringTable.Error,
        _StringTable.ErrorNumber,

        # Bing Shopping Campaigns
        _StringTable.IsExcluded,
        _StringTable.ParentAdGroupCriterionId,
        _StringTable.CampaignType,
        _StringTable.CampaignPriority,
        _StringTable.LocalInventoryAdsEnabled,

        # experiment
        _StringTable.TrafficSplitPercent,
        _StringTable.BaseCampaignId,
        _StringTable.ExperimentCampaignId,
        _StringTable.ExperimentId,
        _StringTable.ExperimentType,

        #CoOp
        _StringTable.BidOption,
        _StringTable.BidBoostValue,
        _StringTable.MaximumBid,

        # V10 added
        _StringTable.FieldPath,

        # Upgrade Url
        _StringTable.FinalUrl,
        _StringTable.FinalMobileUrl,
        _StringTable.TrackingTemplate,
        _StringTable.CustomParameter,

        # Review Ad Extension
        _StringTable.IsExact,
        _StringTable.Source,
        _StringTable.Url,

        # Price Ad Extension
        _StringTable.PriceExtensionType,
        _StringTable.Header1,
        _StringTable.Header2,
        _StringTable.Header3,
        _StringTable.Header4,
        _StringTable.Header5,
        _StringTable.Header6,
        _StringTable.Header7,
        _StringTable.Header8,
        _StringTable.PriceDescription1,
        _StringTable.PriceDescription2,
        _StringTable.PriceDescription3,
        _StringTable.PriceDescription4,
        _StringTable.PriceDescription5,
        _StringTable.PriceDescription6,
        _StringTable.PriceDescription7,
        _StringTable.PriceDescription8,
        _StringTable.FinalUrl1,
        _StringTable.FinalUrl2,
        _StringTable.FinalUrl3,
        _StringTable.FinalUrl4,
        _StringTable.FinalUrl5,
        _StringTable.FinalUrl6,
        _StringTable.FinalUrl7,
        _StringTable.FinalUrl8,
        _StringTable.FinalMobileUrl1,
        _StringTable.FinalMobileUrl2,
        _StringTable.FinalMobileUrl3,
        _StringTable.FinalMobileUrl4,
        _StringTable.FinalMobileUrl5,
        _StringTable.FinalMobileUrl6,
        _StringTable.FinalMobileUrl7,
        _StringTable.FinalMobileUrl8,
        _StringTable.Price1,
        _StringTable.Price2,
        _StringTable.Price3,
        _StringTable.Price4,
        _StringTable.Price5,
        _StringTable.Price6,
        _StringTable.Price7,
        _StringTable.Price8,
        _StringTable.CurrencyCode1,
        _StringTable.CurrencyCode2,
        _StringTable.CurrencyCode3,
        _StringTable.CurrencyCode4,
        _StringTable.CurrencyCode5,
        _StringTable.CurrencyCode6,
        _StringTable.CurrencyCode7,
        _StringTable.CurrencyCode8,
        _StringTable.PriceUnit1,
        _StringTable.PriceUnit2,
        _StringTable.PriceUnit3,
        _StringTable.PriceUnit4,
        _StringTable.PriceUnit5,
        _StringTable.PriceUnit6,
        _StringTable.PriceUnit7,
        _StringTable.PriceUnit8,
        _StringTable.PriceQualifier1,
        _StringTable.PriceQualifier2,
        _StringTable.PriceQualifier3,
        _StringTable.PriceQualifier4,
        _StringTable.PriceQualifier5,
        _StringTable.PriceQualifier6,
        _StringTable.PriceQualifier7,
        _StringTable.PriceQualifier8,

        # Bid Strategy
        _StringTable.BidStrategyId,
        _StringTable.BidStrategyName,
        _StringTable.BidStrategyType,
        _StringTable.BidStrategyMaxCpc,
        _StringTable.BidStrategyTargetCpa,
        _StringTable.BidStrategyTargetRoas,
        _StringTable.InheritedBidStrategyType,
        _StringTable.BidStrategyTargetAdPosition,
        _StringTable.BidStrategyTargetImpressionShare,
        _StringTable.BidStrategyCommissionRate,
        _StringTable.BidStrategyPercentMaxCpc,
        _StringTable.BidStrategyTargetCostPerSale,

        # Ad Format Preference
        _StringTable.AdFormatPreference,

        # Remarketing
        _StringTable.Audience,
        _StringTable.Description,
        _StringTable.MembershipDuration,
        _StringTable.Scope,
        _StringTable.TagId,
        _StringTable.AudienceId,
        _StringTable.TargetSetting,
        _StringTable.RemarketingRule,
        _StringTable.AudienceSearchSize,
        _StringTable.AudienceNetworkSize,
        _StringTable.SupportedCampaignTypes,
        _StringTable.ProductAudienceType,
        _StringTable.SourceId,
        _StringTable.CombinationRule,
        _StringTable.EntityType,
        _StringTable.ImpressionCampaignId,
        _StringTable.ImpressionAdGroupId,


        # Expanded Text Ad
        _StringTable.TitlePart1,
        _StringTable.TitlePart2,
        _StringTable.TitlePart3,
        _StringTable.Path1,
        _StringTable.Path2,
        _StringTable.Domain,

        # Responsive Ad
        _StringTable.CallToAction,
        _StringTable.Headline,
        _StringTable.Images,
        _StringTable.LandscapeImageMediaId,
        _StringTable.LandscapeLogoMediaId,
        _StringTable.LongHeadline,
        _StringTable.LongHeadlines,
        _StringTable.SquareImageMediaId,
        _StringTable.SquareLogoMediaId,
        _StringTable.ImpressionTrackingUrls,
        _StringTable.Headlines,
        _StringTable.Descriptions,
        _StringTable.CallToActionLanguage,
        _StringTable.Videos,

        # Ad Scheduling
        _StringTable.AdSchedule,

        #UseSearcherTimeZone
        _StringTable.UseSearcherTimeZone,
        _StringTable.AdScheduleUseSearcherTimeZone,

        # Action ad extension
        _StringTable.ActionType,
        _StringTable.ActionText,

        # Promotion Ad Extension
        _StringTable.PromotionTarget,
        _StringTable.DiscountModifier,
        _StringTable.PercentOff,
        _StringTable.MoneyAmountOff,
        _StringTable.PromotionCode,
        _StringTable.OrdersOverAmount,
        _StringTable.Occasion,
        _StringTable.PromotionStart,
        _StringTable.PromotionEnd,
        _StringTable.CurrencyCode,

        # Dynamic Search Ads
        _StringTable.DomainLanguage,
        _StringTable.DynamicAdTargetCondition1,
        _StringTable.DynamicAdTargetValue1,
        _StringTable.DynamicAdTargetCondition2,
        _StringTable.DynamicAdTargetValue2,
        _StringTable.DynamicAdTargetCondition3,
        _StringTable.DynamicAdTargetValue3,
        _StringTable.DynamicAdTargetConditionOperator1,
        _StringTable.DynamicAdTargetConditionOperator2,
        _StringTable.DynamicAdTargetConditionOperator3,
        _StringTable.PageFeedIds,
        _StringTable.FeedId,

        # Labels
        _StringTable.ColorCode,
        _StringTable.Label,

        # Offline Conversions
        _StringTable.ConversionCurrencyCode,
        _StringTable.ConversionName,
        _StringTable.ConversionTime,
        _StringTable.ConversionValue,
        _StringTable.MicrosoftClickId,
        _StringTable.AdjustmentValue,
        _StringTable.AdjustmentTime,
        _StringTable.AdjustmentCurrencyCode,
        _StringTable.AdjustmentType,
        _StringTable.ExternalAttributionCredit,
        _StringTable.ExternalAttributionModel,
        _StringTable.HashedEmailAddress,
        _StringTable.HashedPhoneNumber,

        # Account
        _StringTable.MSCLKIDAutoTaggingEnabled,
        _StringTable.IncludeViewThroughConversions,
        _StringTable.ProfileExpansionEnabled,
        _StringTable.AdClickParallelTracking,
        _StringTable.AutoApplyRecommendations,
        _StringTable.AllowImageAutoRetrieve,
        _StringTable.BusinessAttributes,
        _StringTable.FinalUrlSuffix,

        # Feeds
        _StringTable.CustomAttributes,
        _StringTable.FeedName,
        _StringTable.PhysicalIntent,
        _StringTable.TargetAdGroupId,
        _StringTable.TargetCampaignId,
        _StringTable.Schedule,

        # Campaign Conversion Goal
        _StringTable.GoalId,

        # RSA AdCustomizer
        _StringTable.AdCustomizerDataType,
        _StringTable.AdCustomizerAttributeValue,

        # Hotel Ad
        _StringTable.MinTargetValue,
        _StringTable.MaxTargetValue,

        # PMax
        _StringTable.Audiences,
        _StringTable.AssetGroup,
        _StringTable.AudienceGroup,
        _StringTable.AgeRanges,
        _StringTable.GenderTypes,
        _StringTable.ParentListingGroupId,
        _StringTable.AudienceGroupName,
        _StringTable.CampaignNegativeWebpage,
        _StringTable.AssetGroupListingGroup,
        _StringTable.AudienceGroupAssetGroupAssociation,
        _StringTable.AutoGeneratedTextOptOut,
        _StringTable.AutoGeneratedImageOptOut,
        _StringTable.CostPerSaleOptOut,
        _StringTable.SearchTheme,

        # Seasonality Adjustment
        _StringTable.SeasonalityAdjustment,
        _StringTable.DataExclusion,
        _StringTable.DeviceType,
        _StringTable.CampaignAssociations,

        # DNV Serving on MSAN
        _StringTable.ShouldServeOnMSAN,

        # Goal
        _StringTable.AttributionModelType,
        _StringTable.ConversionWindowInMinutes,
        _StringTable.CountType,
        _StringTable.ExcludeFromBidding,
        _StringTable.GoalCategory,
        _StringTable.IsEnhancedConversionsEnabled,
        _StringTable.RevenueType,
        _StringTable.RevenueValue,
        _StringTable.TrackingStatus,
        _StringTable.ViewThroughConversionWindowInMinutes,
        _StringTable.MinimumDurationInSecond,
        _StringTable.ActionExpression,
        _StringTable.ActionOperator,
        _StringTable.CategoryExpression,
        _StringTable.CategoryOperator,
        _StringTable.LabelExpression,
        _StringTable.LabelOperator,
        _StringTable.EventValue,
        _StringTable.EventValueOperator,
        _StringTable.IsExternallyAttributed,
        _StringTable.MinimumPagesViewed,
        _StringTable.UrlExpression,
        _StringTable.UrlOperator,

        # Brand List
        _StringTable.BrandId,
        _StringTable.BrandUrl,
        _StringTable.BrandName,
        _StringTable.StatusDateTime,

        # Asset Group Url Target
        _StringTable.AssetGroupTargetCondition1,
        _StringTable.AssetGroupTargetCondition2,
        _StringTable.AssetGroupTargetCondition3,
        _StringTable.AssetGroupTargetConditionOperator1,
        _StringTable.AssetGroupTargetConditionOperator2,
        _StringTable.AssetGroupTargetConditionOperator3,
        _StringTable.AssetGroupTargetValue1,
        _StringTable.AssetGroupTargetValue2,
        _StringTable.AssetGroupTargetValue3,

        # New Customer Acquisition Goal
        _StringTable.AdditionalConversionValue,
        _StringTable.NewCustomerAcquisitionGoalId,
        _StringTable.NewCustomerAcquisitionBidOnlyMode,

    ]

    @staticmethod
    def get_mappings():
        return _CsvHeaders.COLUMN_INDEX_MAP

    @staticmethod
    def initialize_map():
        return dict(zip(_CsvHeaders.HEADERS, range(len(_CsvHeaders.HEADERS))))


_CsvHeaders.COLUMN_INDEX_MAP = _CsvHeaders.initialize_map()
